@@ -16,12 +16,17 @@
 import io
 import logging
 import os
+import sys
 
-import six
+from osc_lib import utils
 from six.moves import urllib
 
 from openstackclient.api import api
-from openstackclient.common import utils
+
+
+GLOBAL_READ_ACL = ".r:*"
+LIST_CONTENTS_ACL = ".rlistings"
+PUBLIC_CONTAINER_ACLS = [GLOBAL_READ_ACL, LIST_CONTENTS_ACL]
 
 
 class APIv1(api.BaseAPI):
@@ -33,15 +38,32 @@ class APIv1(api.BaseAPI):
     def container_create(
         self,
         container=None,
+        public=False,
+        storage_policy=None
     ):
         """Create a container
 
         :param string container:
             name of container to create
+        :param bool public:
+            Boolean value indicating if the container should be publicly
+            readable. Defaults to False.
+        :param string storage_policy:
+            Name of the a specific storage policy to use. If not specified
+            swift will use its default storage policy.
         :returns:
             dict of returned headers
         """
-        response = self.create(urllib.parse.quote(container), method='PUT')
+
+        headers = {}
+        if public:
+            headers['x-container-read'] = ",".join(PUBLIC_CONTAINER_ACLS)
+        if storage_policy is not None:
+            headers['x-storage-policy'] = storage_policy
+
+        response = self.create(
+            urllib.parse.quote(container), method='PUT', headers=headers)
+
         data = {
             'account': self._find_account_id(),
             'container': container,
@@ -65,7 +87,7 @@ class APIv1(api.BaseAPI):
 
     def container_list(
         self,
-        all_data=False,
+        full_listing=False,
         limit=None,
         marker=None,
         end_marker=None,
@@ -74,7 +96,7 @@ class APIv1(api.BaseAPI):
     ):
         """Get containers in an account
 
-        :param boolean all_data:
+        :param boolean full_listing:
             if True, return a full listing, else returns a max of
             10000 listings
         :param integer limit:
@@ -91,7 +113,7 @@ class APIv1(api.BaseAPI):
 
         params['format'] = 'json'
 
-        if all_data:
+        if full_listing:
             data = listing = self.container_list(
                 limit=limit,
                 marker=marker,
@@ -173,7 +195,8 @@ class APIv1(api.BaseAPI):
             'object_count': response.headers.get(
                 'x-container-object-count'
             ),
-            'bytes_used': response.headers.get('x-container-bytes-used')
+            'bytes_used': response.headers.get('x-container-bytes-used'),
+            'storage_policy': response.headers.get('x-storage-policy'),
         }
 
         if 'x-container-read' in response.headers:
@@ -214,6 +237,7 @@ class APIv1(api.BaseAPI):
         self,
         container=None,
         object=None,
+        name=None,
     ):
         """Create an object inside a container
 
@@ -221,6 +245,8 @@ class APIv1(api.BaseAPI):
             name of container to store object
         :param string object:
             local path to object
+        :param string name:
+            name of object to create
         :returns:
             dict of returned headers
         """
@@ -229,8 +255,12 @@ class APIv1(api.BaseAPI):
             # TODO(dtroyer): What exception to raise here?
             return {}
 
+        # For uploading a file, if name is provided then set it as the
+        # object's name in the container.
+        object_name_str = name if name else object
+
         full_url = "%s/%s" % (urllib.parse.quote(container),
-                              urllib.parse.quote(object))
+                              urllib.parse.quote(object_name_str))
         with io.open(object, 'rb') as f:
             response = self.create(
                 full_url,
@@ -240,7 +270,7 @@ class APIv1(api.BaseAPI):
         data = {
             'account': self._find_account_id(),
             'container': container,
-            'object': object,
+            'object': object_name_str,
             'x-trans-id': response.headers.get('X-Trans-Id'),
             'etag': response.headers.get('Etag'),
         }
@@ -269,7 +299,7 @@ class APIv1(api.BaseAPI):
     def object_list(
         self,
         container=None,
-        all_data=False,
+        full_listing=False,
         limit=None,
         marker=None,
         end_marker=None,
@@ -281,7 +311,7 @@ class APIv1(api.BaseAPI):
 
         :param string container:
             container name to get a listing for
-        :param boolean all_data:
+        :param boolean full_listing:
             if True, return a full listing, else returns a max of
             10000 listings
         :param integer limit:
@@ -298,11 +328,11 @@ class APIv1(api.BaseAPI):
             headers will be a dict and all header names will be lowercase.
         """
 
-        if container is None or object is None:
+        if container is None:
             return None
 
         params['format'] = 'json'
-        if all_data:
+        if full_listing:
             data = listing = self.object_list(
                 container=container,
                 limit=limit,
@@ -369,12 +399,17 @@ class APIv1(api.BaseAPI):
             stream=True,
         )
         if response.status_code == 200:
-            if not os.path.exists(os.path.dirname(file)):
-                if len(os.path.dirname(file)) > 0:
-                    os.makedirs(os.path.dirname(file))
-            with open(file, 'wb') as f:
-                for chunk in response.iter_content():
-                    f.write(chunk)
+            if file == '-':
+                with os.fdopen(sys.stdout.fileno(), 'wb') as f:
+                    for chunk in response.iter_content(64 * 1024):
+                        f.write(chunk)
+            else:
+                if not os.path.exists(os.path.dirname(file)):
+                    if len(os.path.dirname(file)) > 0:
+                        os.makedirs(os.path.dirname(file))
+                with open(file, 'wb') as f:
+                    for chunk in response.iter_content(64 * 1024):
+                        f.write(chunk)
 
     def object_set(
         self,
@@ -546,7 +581,7 @@ class APIv1(api.BaseAPI):
         log = logging.getLogger(__name__ + '._set_properties')
 
         headers = {}
-        for k, v in six.iteritems(properties):
+        for k, v in properties.items():
             if not utils.is_ascii(k) or not utils.is_ascii(v):
                 log.error('Cannot set property %s to non-ascii value', k)
                 continue
@@ -559,7 +594,7 @@ class APIv1(api.BaseAPI):
         # Add in properties as a top level key, this is consistent with other
         # OSC commands
         properties = {}
-        for k, v in six.iteritems(headers):
+        for k, v in headers.items():
             if k.lower().startswith(header_tag):
                 properties[k[len(header_tag):]] = v
         return properties
