@@ -18,11 +18,11 @@
 
 import logging
 
+from osc_lib.cli import format_columns
 from osc_lib.cli import parseractions
 from osc_lib.command import command
 from osc_lib import exceptions
 from osc_lib import utils
-import six
 
 from openstackclient.i18n import _
 
@@ -31,7 +31,7 @@ LOG = logging.getLogger(__name__)
 
 
 class AddAggregateHost(command.ShowOne):
-    """Add host to aggregate"""
+    _description = _("Add host to aggregate")
 
     def get_parser(self, prog_name):
         parser = super(AddAggregateHost, self).get_parser(prog_name)
@@ -58,11 +58,20 @@ class AddAggregateHost(command.ShowOne):
 
         info = {}
         info.update(data._info)
-        return zip(*sorted(six.iteritems(info)))
+
+        # Special mapping for columns to make the output easier to read:
+        # 'metadata' --> 'properties'
+        info.update(
+            {
+                'hosts': format_columns.ListColumn(info.pop('hosts')),
+                'properties': format_columns.DictColumn(info.pop('metadata')),
+            },
+        )
+        return zip(*sorted(info.items()))
 
 
 class CreateAggregate(command.ShowOne):
-    """Create a new aggregate"""
+    _description = _("Create a new aggregate")
 
     def get_parser(self, prog_name):
         parser = super(CreateAggregate, self).get_parser(prog_name)
@@ -101,11 +110,25 @@ class CreateAggregate(command.ShowOne):
                 parsed_args.property,
             )._info)
 
-        return zip(*sorted(six.iteritems(info)))
+        # Special mapping for columns to make the output easier to read:
+        # 'metadata' --> 'properties'
+        hosts = None
+        properties = None
+        if 'hosts' in info.keys():
+            hosts = format_columns.ListColumn(info.pop('hosts'))
+        if 'metadata' in info.keys():
+            properties = format_columns.DictColumn(info.pop('metadata'))
+        info.update(
+            {
+                'hosts': hosts,
+                'properties': properties,
+            },
+        )
+        return zip(*sorted(info.items()))
 
 
 class DeleteAggregate(command.Command):
-    """Delete existing aggregate(s)"""
+    _description = _("Delete existing aggregate(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteAggregate, self).get_parser(prog_name)
@@ -128,8 +151,8 @@ class DeleteAggregate(command.Command):
             except Exception as e:
                 result += 1
                 LOG.error(_("Failed to delete aggregate with name or "
-                          "ID '%(aggregate)s': %(e)s")
-                          % {'aggregate': a, 'e': e})
+                          "ID '%(aggregate)s': %(e)s"),
+                          {'aggregate': a, 'e': e})
 
         if result > 0:
             total = len(parsed_args.aggregate)
@@ -139,7 +162,7 @@ class DeleteAggregate(command.Command):
 
 
 class ListAggregate(command.Lister):
-    """List all aggregates"""
+    _description = _("List all aggregates")
 
     def get_parser(self, prog_name):
         parser = super(ListAggregate, self).get_parser(prog_name)
@@ -184,11 +207,15 @@ class ListAggregate(command.Lister):
         return (column_headers,
                 (utils.get_item_properties(
                     s, columns,
+                    formatters={
+                        'Hosts': format_columns.ListColumn,
+                        'Metadata': format_columns.DictColumn,
+                    },
                 ) for s in data))
 
 
 class RemoveAggregateHost(command.ShowOne):
-    """Remove host from aggregate"""
+    _description = _("Remove host from aggregate")
 
     def get_parser(self, prog_name):
         parser = super(RemoveAggregateHost, self).get_parser(prog_name)
@@ -218,11 +245,20 @@ class RemoveAggregateHost(command.ShowOne):
 
         info = {}
         info.update(data._info)
-        return zip(*sorted(six.iteritems(info)))
+
+        # Special mapping for columns to make the output easier to read:
+        # 'metadata' --> 'properties'
+        info.update(
+            {
+                'hosts': format_columns.ListColumn(info.pop('hosts')),
+                'properties': format_columns.DictColumn(info.pop('metadata')),
+            },
+        )
+        return zip(*sorted(info.items()))
 
 
 class SetAggregate(command.Command):
-    """Set aggregate properties"""
+    _description = _("Set aggregate properties")
 
     def get_parser(self, prog_name):
         parser = super(SetAggregate, self).get_parser(prog_name)
@@ -248,6 +284,14 @@ class SetAggregate(command.Command):
             help=_("Property to set on <aggregate> "
                    "(repeat option to set multiple properties)")
         )
+        parser.add_argument(
+            "--no-property",
+            dest="no_property",
+            action="store_true",
+            help=_("Remove all properties from <aggregate> "
+                   "(specify both --property and --no-property to "
+                   "overwrite the current properties)"),
+        )
         return parser
 
     def take_action(self, parsed_args):
@@ -269,15 +313,28 @@ class SetAggregate(command.Command):
                 kwargs
             )
 
+        set_property = {}
+        if parsed_args.no_property:
+            # NOTE(RuiChen): "availability_zone" is removed from response of
+            #                aggregate show and create commands, don't see it
+            #                anywhere, so pop it, avoid the unexpected server
+            #                exception(can't unset the availability zone from
+            #                aggregate metadata in nova).
+            set_property.update({key: None
+                                 for key in aggregate.metadata.keys()
+                                 if key != 'availability_zone'})
         if parsed_args.property:
+            set_property.update(parsed_args.property)
+
+        if set_property:
             compute_client.aggregates.set_metadata(
                 aggregate,
-                parsed_args.property
+                set_property
             )
 
 
 class ShowAggregate(command.ShowOne):
-    """Display aggregate details"""
+    _description = _("Display aggregate details")
 
     def get_parser(self, prog_name):
         parser = super(ShowAggregate, self).get_parser(prog_name)
@@ -303,17 +360,22 @@ class ShowAggregate(command.ShowOne):
         # 'metadata' --> 'properties'
         data._info.update(
             {
-                'properties': utils.format_dict(data._info.pop('metadata')),
+                'hosts': format_columns.ListColumn(
+                    data._info.pop('hosts')
+                ),
+                'properties': format_columns.DictColumn(
+                    data._info.pop('metadata')
+                ),
             },
         )
 
         info = {}
         info.update(data._info)
-        return zip(*sorted(six.iteritems(info)))
+        return zip(*sorted(info.items()))
 
 
 class UnsetAggregate(command.Command):
-    """Unset aggregate properties"""
+    _description = _("Unset aggregate properties")
 
     def get_parser(self, prog_name):
         parser = super(UnsetAggregate, self).get_parser(prog_name)
@@ -326,7 +388,6 @@ class UnsetAggregate(command.Command):
             "--property",
             metavar="<key>",
             action='append',
-            required=True,
             help=_("Property to remove from aggregate "
                    "(repeat option to remove multiple properties)")
         )
@@ -338,6 +399,9 @@ class UnsetAggregate(command.Command):
             compute_client.aggregates,
             parsed_args.aggregate)
 
-        unset_property = {key: None for key in parsed_args.property}
-        compute_client.aggregates.set_metadata(aggregate,
-                                               unset_property)
+        unset_property = {}
+        if parsed_args.property:
+            unset_property.update({key: None for key in parsed_args.property})
+        if unset_property:
+            compute_client.aggregates.set_metadata(aggregate,
+                                                   unset_property)
