@@ -15,40 +15,35 @@
 
 """Compute v2 Console action implementations"""
 
-import logging
-import six
-import sys
+from osc_lib.cli import parseractions
+from osc_lib.command import command
+from osc_lib import utils
 
-from cliff import command
-from cliff import show
-
-from openstackclient.common import utils
+from openstackclient.i18n import _
 
 
 class ShowConsoleLog(command.Command):
-    """Show console-log command"""
-
-    log = logging.getLogger(__name__ + '.ShowConsoleLog')
+    _description = _("Show server's console output")
 
     def get_parser(self, prog_name):
         parser = super(ShowConsoleLog, self).get_parser(prog_name)
         parser.add_argument(
             'server',
             metavar='<server>',
-            help='Name or ID of server to display console log',
+            help=_("Server to show console log (name or ID)")
         )
         parser.add_argument(
             '--lines',
             metavar='<num-lines>',
             type=int,
             default=None,
-            help='Number of lines to display from the end of the log '
-                 '(default=all)',
+            action=parseractions.NonNegativeAction,
+            help=_("Number of lines to display from the end of the log "
+                   "(default=all)")
         )
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         compute_client = self.app.client_manager.compute
 
         server = utils.find_resource(
@@ -62,21 +57,21 @@ class ShowConsoleLog(command.Command):
             length += 1
 
         data = server.get_console_output(length=length)
-        sys.stdout.write(data)
-        return
+
+        if data and data[-1] != '\n':
+            data += '\n'
+        self.app.stdout.write(data)
 
 
-class ShowConsoleURL(show.ShowOne):
-    """Show console-url command"""
-
-    log = logging.getLogger(__name__ + '.ShowConsoleURL')
+class ShowConsoleURL(command.ShowOne):
+    _description = _("Show server's remote console URL")
 
     def get_parser(self, prog_name):
         parser = super(ShowConsoleURL, self).get_parser(prog_name)
         parser.add_argument(
             'server',
             metavar='<server>',
-            help='Name or ID of server to display console log',
+            help=_("Server to show URL (name or ID)")
         )
         type_group = parser.add_mutually_exclusive_group()
         type_group.add_argument(
@@ -85,40 +80,61 @@ class ShowConsoleURL(show.ShowOne):
             action='store_const',
             const='novnc',
             default='novnc',
-            help='Show noVNC console URL (default)',
+            help=_("Show noVNC console URL (default)")
         )
         type_group.add_argument(
             '--xvpvnc',
             dest='url_type',
             action='store_const',
             const='xvpvnc',
-            help='Show xpvnc console URL',
+            help=_("Show xvpvnc console URL")
         )
         type_group.add_argument(
             '--spice',
             dest='url_type',
             action='store_const',
-            const='spice',
-            help='Show SPICE console URL',
+            const='spice-html5',
+            help=_("Show SPICE console URL")
+        )
+        type_group.add_argument(
+            '--rdp',
+            dest='url_type',
+            action='store_const',
+            const='rdp-html5',
+            help=_("Show RDP console URL"),
+        )
+        type_group.add_argument(
+            '--serial',
+            dest='url_type',
+            action='store_const',
+            const='serial',
+            help=_("Show serial console URL"),
+        )
+        type_group.add_argument(
+            '--mks',
+            dest='url_type',
+            action='store_const',
+            const='webmks',
+            help=_("Show WebMKS console URL"),
         )
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         compute_client = self.app.client_manager.compute
         server = utils.find_resource(
             compute_client.servers,
             parsed_args.server,
         )
 
-        if parsed_args.url_type in ['novnc', 'xvpvnc']:
-            data = server.get_vnc_console(parsed_args.url_type)
-        if parsed_args.url_type in ['spice']:
-            data = server.get_spice_console(parsed_args.url_type)
-
+        data = server.get_console_url(parsed_args.url_type)
         if not data:
             return ({}, {})
 
         info = {}
-        info.update(data['console'])
-        return zip(*sorted(six.iteritems(info)))
+        # NOTE(Rui Chen): Return 'remote_console' in compute microversion API
+        #                 2.6 and later, return 'console' in compute
+        #                 microversion API from 2.0 to 2.5, do compatibility
+        #                 handle for different microversion API.
+        console_data = data.get('remote_console', data.get('console'))
+        info.update(console_data)
+        return zip(*sorted(info.items()))
