@@ -15,18 +15,21 @@
 
 """Identity v2 Role action implementations"""
 
-import six
+import logging
 
 from keystoneauth1 import exceptions as ks_exc
+from osc_lib.command import command
+from osc_lib import exceptions
+from osc_lib import utils
 
-from openstackclient.common import command
-from openstackclient.common import exceptions
-from openstackclient.common import utils
-from openstackclient.i18n import _  # noqa
+from openstackclient.i18n import _
+
+
+LOG = logging.getLogger(__name__)
 
 
 class AddRole(command.ShowOne):
-    """Add role to project:user"""
+    _description = _("Add role to project:user")
 
     def get_parser(self, prog_name):
         parser = super(AddRole, self).get_parser(prog_name)
@@ -65,11 +68,11 @@ class AddRole(command.ShowOne):
 
         info = {}
         info.update(role._info)
-        return zip(*sorted(six.iteritems(info)))
+        return zip(*sorted(info.items()))
 
 
 class CreateRole(command.ShowOne):
-    """Create new role"""
+    _description = _("Create new role")
 
     def get_parser(self, prog_name):
         parser = super(CreateRole, self).get_parser(prog_name)
@@ -89,23 +92,23 @@ class CreateRole(command.ShowOne):
         identity_client = self.app.client_manager.identity
         try:
             role = identity_client.roles.create(parsed_args.role_name)
-        except ks_exc.Conflict as e:
+        except ks_exc.Conflict:
             if parsed_args.or_show:
                 role = utils.find_resource(
                     identity_client.roles,
                     parsed_args.role_name,
                 )
-                self.log.info('Returning existing role %s', role.name)
+                LOG.info(_('Returning existing role %s'), role.name)
             else:
-                raise e
+                raise
 
         info = {}
         info.update(role._info)
-        return zip(*sorted(six.iteritems(info)))
+        return zip(*sorted(info.items()))
 
 
 class DeleteRole(command.Command):
-    """Delete role(s)"""
+    _description = _("Delete role(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteRole, self).get_parser(prog_name)
@@ -120,150 +123,35 @@ class DeleteRole(command.Command):
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
 
+        errors = 0
         for role in parsed_args.roles:
-            role_obj = utils.find_resource(
-                identity_client.roles,
-                role,
-            )
-            identity_client.roles.delete(role_obj.id)
-        return
+            try:
+                role_obj = utils.find_resource(
+                    identity_client.roles,
+                    role,
+                )
+                identity_client.roles.delete(role_obj.id)
+            except Exception as e:
+                errors += 1
+                LOG.error(_("Failed to delete role with "
+                          "name or ID '%(role)s': %(e)s"),
+                          {'role': role, 'e': e})
+
+        if errors > 0:
+            total = len(parsed_args.roles)
+            msg = (_("%(errors)s of %(total)s roles failed "
+                   "to delete.") % {'errors': errors, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
 class ListRole(command.Lister):
-    """List roles"""
-
-    def get_parser(self, prog_name):
-        parser = super(ListRole, self).get_parser(prog_name)
-        parser.add_argument(
-            '--project',
-            metavar='<project>',
-            help='Filter roles by <project> (name or ID)',
-        )
-        parser.add_argument(
-            '--user',
-            metavar='<user>',
-            help='Filter roles by <user> (name or ID)',
-        )
-        return parser
+    _description = _("List roles")
 
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
-        auth_ref = self.app.client_manager.auth_ref
 
-        # No user or project specified, list all roles in the system
-        if not parsed_args.user and not parsed_args.project:
-            columns = ('ID', 'Name')
-            data = identity_client.roles.list()
-        elif parsed_args.user and parsed_args.project:
-            user = utils.find_resource(
-                identity_client.users,
-                parsed_args.user,
-            )
-            project = utils.find_resource(
-                identity_client.projects,
-                parsed_args.project,
-            )
-            data = identity_client.roles.roles_for_user(user.id, project.id)
-
-        elif parsed_args.user:
-            user = utils.find_resource(
-                identity_client.users,
-                parsed_args.user,
-            )
-            if self.app.client_manager.auth_ref:
-                project = utils.find_resource(
-                    identity_client.projects,
-                    auth_ref.project_id
-                )
-            else:
-                msg = _("Project must be specified")
-                raise exceptions.CommandError(msg)
-            data = identity_client.roles.roles_for_user(user.id, project.id)
-        elif parsed_args.project:
-            project = utils.find_resource(
-                identity_client.projects,
-                parsed_args.project,
-            )
-            if self.app.client_manager.auth_ref:
-                user = utils.find_resource(
-                    identity_client.users,
-                    auth_ref.user_id
-                )
-            else:
-                msg = _("User must be specified")
-                raise exceptions.CommandError(msg)
-            data = identity_client.roles.roles_for_user(user.id, project.id)
-
-        if parsed_args.user or parsed_args.project:
-            columns = ('ID', 'Name', 'Project', 'User')
-            for user_role in data:
-                user_role.user = user.name
-                user_role.project = project.name
-
-        return (columns,
-                (utils.get_item_properties(
-                    s, columns,
-                    formatters={},
-                ) for s in data))
-
-
-class ListUserRole(command.Lister):
-    """List user-role assignments"""
-
-    def get_parser(self, prog_name):
-        parser = super(ListUserRole, self).get_parser(prog_name)
-        parser.add_argument(
-            'user',
-            metavar='<user>',
-            nargs='?',
-            help=_('User to list (name or ID)'),
-        )
-        parser.add_argument(
-            '--project',
-            metavar='<project>',
-            help=_('Filter users by <project> (name or ID)'),
-        )
-        return parser
-
-    def take_action(self, parsed_args):
-        identity_client = self.app.client_manager.identity
-        auth_ref = self.app.client_manager.auth_ref
-
-        # Project and user are required, if not included in command args
-        # default to the values used for authentication.  For token-flow
-        # authentication they must be included on the command line.
-        if not parsed_args.project:
-            if self.app.client_manager.auth_ref:
-                parsed_args.project = auth_ref.project_id
-            else:
-                msg = _("Project must be specified")
-                raise exceptions.CommandError(msg)
-        if not parsed_args.user:
-            if self.app.client_manager.auth_ref:
-                parsed_args.user = auth_ref.user_id
-            else:
-                msg = _("User must be specified")
-                raise exceptions.CommandError(msg)
-
-        project = utils.find_resource(
-            identity_client.tenants,
-            parsed_args.project,
-        )
-        user = utils.find_resource(identity_client.users, parsed_args.user)
-
-        data = identity_client.roles.roles_for_user(user.id, project.id)
-
-        columns = (
-            'ID',
-            'Name',
-            'Project',
-            'User',
-        )
-
-        # Add the names to the output even though they will be constant
-        for role in data:
-            role.user = user.name
-            role.project = project.name
+        columns = ('ID', 'Name')
+        data = identity_client.roles.list()
 
         return (columns,
                 (utils.get_item_properties(
@@ -273,7 +161,7 @@ class ListUserRole(command.Lister):
 
 
 class RemoveRole(command.Command):
-    """Remove role from project : user"""
+    _description = _("Remove role from project : user")
 
     def get_parser(self, prog_name):
         parser = super(RemoveRole, self).get_parser(prog_name)
@@ -311,7 +199,7 @@ class RemoveRole(command.Command):
 
 
 class ShowRole(command.ShowOne):
-    """Display role details"""
+    _description = _("Display role details")
 
     def get_parser(self, prog_name):
         parser = super(ShowRole, self).get_parser(prog_name)
@@ -328,4 +216,4 @@ class ShowRole(command.ShowOne):
 
         info = {}
         info.update(role._info)
-        return zip(*sorted(six.iteritems(info)))
+        return zip(*sorted(info.items()))
