@@ -16,89 +16,117 @@
 """Identity v3 Domain action implementations"""
 
 import logging
-import six
-import sys
 
-from cliff import command
-from cliff import lister
-from cliff import show
+from keystoneauth1 import exceptions as ks_exc
+from osc_lib.command import command
+from osc_lib import exceptions
+from osc_lib import utils
 
-from openstackclient.common import utils
+from openstackclient.i18n import _
+from openstackclient.identity import common
 
 
-class CreateDomain(show.ShowOne):
-    """Create domain command"""
+LOG = logging.getLogger(__name__)
 
-    log = logging.getLogger(__name__ + '.CreateDomain')
+
+class CreateDomain(command.ShowOne):
+    _description = _("Create new domain")
 
     def get_parser(self, prog_name):
         parser = super(CreateDomain, self).get_parser(prog_name)
         parser.add_argument(
             'name',
             metavar='<domain-name>',
-            help='New domain name',
+            help=_('New domain name'),
         )
         parser.add_argument(
             '--description',
-            metavar='<domain-description>',
-            help='New domain description',
+            metavar='<description>',
+            help=_('New domain description'),
         )
         enable_group = parser.add_mutually_exclusive_group()
         enable_group.add_argument(
             '--enable',
-            dest='enabled',
             action='store_true',
-            default=True,
-            help='Enable domain')
+            help=_('Enable domain (default)'),
+        )
         enable_group.add_argument(
             '--disable',
-            dest='enabled',
-            action='store_false',
-            help='Disable domain')
+            action='store_true',
+            help=_('Disable domain'),
+        )
+        parser.add_argument(
+            '--or-show',
+            action='store_true',
+            help=_('Return existing domain'),
+        )
+        common.add_resource_option_to_parser(parser)
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
-        domain = identity_client.domains.create(
-            name=parsed_args.name,
-            description=parsed_args.description,
-            enabled=parsed_args.enabled,
-        )
 
-        return zip(*sorted(six.iteritems(domain._info)))
+        enabled = True
+        if parsed_args.disable:
+            enabled = False
+
+        options = common.get_immutable_options(parsed_args)
+
+        try:
+            domain = identity_client.domains.create(
+                name=parsed_args.name,
+                description=parsed_args.description,
+                options=options,
+                enabled=enabled,
+            )
+        except ks_exc.Conflict:
+            if parsed_args.or_show:
+                domain = utils.find_resource(identity_client.domains,
+                                             parsed_args.name)
+                LOG.info(_('Returning existing domain %s'), domain.name)
+            else:
+                raise
+
+        domain._info.pop('links')
+        return zip(*sorted(domain._info.items()))
 
 
 class DeleteDomain(command.Command):
-    """Delete domain command"""
-
-    log = logging.getLogger(__name__ + '.DeleteDomain')
+    _description = _("Delete domain(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteDomain, self).get_parser(prog_name)
         parser.add_argument(
             'domain',
             metavar='<domain>',
-            help='Name or ID of domain to delete',
+            nargs='+',
+            help=_('Domain(s) to delete (name or ID)'),
         )
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
-        domain = utils.find_resource(identity_client.domains,
-                                     parsed_args.domain)
-        identity_client.domains.delete(domain.id)
-        return
+        result = 0
+        for i in parsed_args.domain:
+            try:
+                domain = utils.find_resource(identity_client.domains, i)
+                identity_client.domains.delete(domain.id)
+            except Exception as e:
+                result += 1
+                LOG.error(_("Failed to delete domain with name or "
+                          "ID '%(domain)s': %(e)s"), {'domain': i, 'e': e})
+
+        if result > 0:
+            total = len(parsed_args.domain)
+            msg = (_("%(result)s of %(total)s domains failed "
+                   "to delete.") % {'result': result, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
-class ListDomain(lister.Lister):
-    """List domain command"""
-
-    log = logging.getLogger(__name__ + '.ListDomain')
+class ListDomain(command.Lister):
+    _description = _("List domains")
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         columns = ('ID', 'Name', 'Enabled', 'Description')
         data = self.app.client_manager.identity.domains.list()
         return (columns,
@@ -109,45 +137,40 @@ class ListDomain(lister.Lister):
 
 
 class SetDomain(command.Command):
-    """Set domain command"""
-
-    log = logging.getLogger(__name__ + '.SetDomain')
+    _description = _("Set domain properties")
 
     def get_parser(self, prog_name):
         parser = super(SetDomain, self).get_parser(prog_name)
         parser.add_argument(
             'domain',
             metavar='<domain>',
-            help='Name or ID of domain to change',
+            help=_('Domain to modify (name or ID)'),
         )
         parser.add_argument(
             '--name',
-            metavar='<new-domain-name>',
-            help='New domain name',
+            metavar='<name>',
+            help=_('New domain name'),
         )
         parser.add_argument(
             '--description',
-            metavar='<domain-description>',
-            help='New domain description',
+            metavar='<description>',
+            help=_('New domain description'),
         )
         enable_group = parser.add_mutually_exclusive_group()
         enable_group.add_argument(
             '--enable',
-            dest='enabled',
             action='store_true',
-            default=True,
-            help='Enable domain (default)',
+            help=_('Enable domain'),
         )
         enable_group.add_argument(
             '--disable',
-            dest='enabled',
-            action='store_false',
-            help='Disable domain',
+            action='store_true',
+            help=_('Disable domain'),
         )
+        common.add_resource_option_to_parser(parser)
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
         domain = utils.find_resource(identity_client.domains,
                                      parsed_args.domain)
@@ -156,34 +179,39 @@ class SetDomain(command.Command):
             kwargs['name'] = parsed_args.name
         if parsed_args.description:
             kwargs['description'] = parsed_args.description
-        if 'enabled' in parsed_args:
-            kwargs['enabled'] = parsed_args.enabled
 
-        if not kwargs:
-            sys.stdout.write("Domain not updated, no arguments present")
-            return
+        if parsed_args.enable:
+            kwargs['enabled'] = True
+        if parsed_args.disable:
+            kwargs['enabled'] = False
+
+        options = common.get_immutable_options(parsed_args)
+        if options:
+            kwargs['options'] = options
+
         identity_client.domains.update(domain.id, **kwargs)
-        return
 
 
-class ShowDomain(show.ShowOne):
-    """Show domain command"""
-
-    log = logging.getLogger(__name__ + '.ShowDomain')
+class ShowDomain(command.ShowOne):
+    _description = _("Display domain details")
 
     def get_parser(self, prog_name):
         parser = super(ShowDomain, self).get_parser(prog_name)
         parser.add_argument(
             'domain',
             metavar='<domain>',
-            help='Name or ID of domain to display',
+            help=_('Domain to display (name or ID)'),
         )
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
-        domain = utils.find_resource(identity_client.domains,
-                                     parsed_args.domain)
 
-        return zip(*sorted(six.iteritems(domain._info)))
+        domain_str = common._get_token_resource(identity_client, 'domain',
+                                                parsed_args.domain)
+
+        domain = utils.find_resource(identity_client.domains,
+                                     domain_str)
+
+        domain._info.pop('links')
+        return zip(*sorted(domain._info.items()))
