@@ -17,182 +17,211 @@
 
 import argparse
 import io
+import logging
 import os
-import six
 import sys
 
+from cliff import columns as cliff_columns
+from osc_lib.api import utils as api_utils
+from osc_lib.cli import format_columns
+from osc_lib.cli import parseractions
+from osc_lib.command import command
+from osc_lib import utils
+
+from openstackclient.common import sdk_utils
+from openstackclient.i18n import _
+
 if os.name == "nt":
     import msvcrt
 else:
     msvcrt = None
 
-from glanceclient.common import utils as gc_utils
-from openstackclient.api import utils as api_utils
-from openstackclient.common import command
-from openstackclient.common import parseractions
-from openstackclient.common import utils
-from openstackclient.i18n import _  # noqa
-
 
+CONTAINER_CHOICES = ["ami", "ari", "aki", "bare", "docker", "ova", "ovf"]
 DEFAULT_CONTAINER_FORMAT = 'bare'
 DEFAULT_DISK_FORMAT = 'raw'
+DISK_CHOICES = ["ami", "ari", "aki", "vhd", "vmdk", "raw", "qcow2", "vhdx",
+                "vdi", "iso", "ploop"]
 
 
-def _format_visibility(data):
-    """Return a formatted visibility string
+LOG = logging.getLogger(__name__)
 
-    :param data:
-        The server's visibility (is_public) status value: True, False
-    :rtype:
-        A string formatted to public/private
-    """
 
-    if data:
-        return 'public'
-    else:
-        return 'private'
+def _get_columns(item):
+    # Trick sdk_utils to return URI attribute
+    column_map = {
+        'is_protected': 'protected',
+        'owner_id': 'owner'
+    }
+    hidden_columns = ['location', 'checksum',
+                      'copy_from', 'created_at', 'status', 'updated_at']
+    return sdk_utils.get_osc_show_columns_for_sdk_resource(
+        item.to_dict(), column_map, hidden_columns)
+
+
+_formatters = {
+}
+
+
+class HumanReadableSizeColumn(cliff_columns.FormattableColumn):
+    def human_readable(self):
+        """Return a formatted visibility string
+
+        :rtype:
+            A string formatted to public/private
+        """
+
+        if self._value:
+            return utils.format_size(self._value)
+        else:
+            return ''
+
+
+class VisibilityColumn(cliff_columns.FormattableColumn):
+    def human_readable(self):
+        """Return a formatted visibility string
+
+        :rtype:
+            A string formatted to public/private
+        """
+
+        if self._value:
+            return 'public'
+        else:
+            return 'private'
 
 
 class CreateImage(command.ShowOne):
-    """Create/upload an image"""
+    _description = _("Create/upload an image")
 
     def get_parser(self, prog_name):
         parser = super(CreateImage, self).get_parser(prog_name)
         parser.add_argument(
             "name",
             metavar="<image-name>",
-            help="New image name",
+            help=_("New image name"),
         )
         parser.add_argument(
             "--id",
             metavar="<id>",
-            help="Image ID to reserve",
+            help=_("Image ID to reserve"),
         )
         parser.add_argument(
             "--store",
             metavar="<store>",
-            help="Upload image to this store",
+            help=_("Upload image to this store"),
         )
         parser.add_argument(
             "--container-format",
             default=DEFAULT_CONTAINER_FORMAT,
             metavar="<container-format>",
-            help="Image container format "
-                 "(default: %s)" % DEFAULT_CONTAINER_FORMAT,
+            choices=CONTAINER_CHOICES,
+            help=(_("Image container format. "
+                    "The supported options are: %(option_list)s. "
+                    "The default format is: %(default_opt)s") %
+                  {'option_list': ', '.join(CONTAINER_CHOICES),
+                   'default_opt': DEFAULT_CONTAINER_FORMAT})
         )
         parser.add_argument(
             "--disk-format",
             default=DEFAULT_DISK_FORMAT,
             metavar="<disk-format>",
-            help="Image disk format "
-                 "(default: %s)" % DEFAULT_DISK_FORMAT,
+            choices=DISK_CHOICES,
+            help=_("Image disk format. The supported options are: %s. "
+                   "The default format is: raw") % ', '.join(DISK_CHOICES)
         )
         parser.add_argument(
             "--size",
             metavar="<size>",
-            help="Image size, in bytes (only used with --location and"
-                 " --copy-from)",
+            help=_("Image size, in bytes (only used with --location and"
+                   " --copy-from)"),
         )
         parser.add_argument(
             "--min-disk",
             metavar="<disk-gb>",
             type=int,
-            help="Minimum disk size needed to boot image, in gigabytes",
+            help=_("Minimum disk size needed to boot image, in gigabytes"),
         )
         parser.add_argument(
             "--min-ram",
             metavar="<ram-mb>",
             type=int,
-            help="Minimum RAM size needed to boot image, in megabytes",
+            help=_("Minimum RAM size needed to boot image, in megabytes"),
         )
         parser.add_argument(
             "--location",
             metavar="<image-url>",
-            help="Download image from an existing URL",
+            help=_("Download image from an existing URL"),
         )
         parser.add_argument(
             "--copy-from",
             metavar="<image-url>",
-            help="Copy image from the data store (similar to --location)",
+            help=_("Copy image from the data store (similar to --location)"),
         )
-        parser.add_argument(
+        source_group = parser.add_mutually_exclusive_group()
+        source_group.add_argument(
             "--file",
             metavar="<file>",
-            help="Upload image from local file",
+            help=_("Upload image from local file"),
         )
-        parser.add_argument(
+        source_group.add_argument(
             "--volume",
             metavar="<volume>",
-            help="Create image from a volume",
+            help=_("Create image from a volume"),
         )
         parser.add_argument(
             "--force",
             dest='force',
             action='store_true',
             default=False,
-            help="Force image creation if volume is in use "
-                 "(only meaningful with --volume)",
+            help=_("Force image creation if volume is in use "
+                   "(only meaningful with --volume)"),
         )
         parser.add_argument(
             "--checksum",
             metavar="<checksum>",
-            help="Image hash used for verification",
+            help=_("Image hash used for verification"),
         )
         protected_group = parser.add_mutually_exclusive_group()
         protected_group.add_argument(
             "--protected",
             action="store_true",
-            help="Prevent image from being deleted",
+            help=_("Prevent image from being deleted"),
         )
         protected_group.add_argument(
             "--unprotected",
             action="store_true",
-            help="Allow image to be deleted (default)",
+            help=_("Allow image to be deleted (default)"),
         )
         public_group = parser.add_mutually_exclusive_group()
         public_group.add_argument(
             "--public",
             action="store_true",
-            help="Image is accessible to the public",
+            help=_("Image is accessible to the public"),
         )
         public_group.add_argument(
             "--private",
             action="store_true",
-            help="Image is inaccessible to the public (default)",
+            help=_("Image is inaccessible to the public (default)"),
         )
         parser.add_argument(
             "--property",
             dest="properties",
             metavar="<key=value>",
             action=parseractions.KeyValueAction,
-            help="Set a property on this image "
-                 "(repeat option to set multiple properties)",
+            help=_("Set a property on this image "
+                   "(repeat option to set multiple properties)"),
         )
-        # NOTE(dtroyer): --owner is deprecated in Jan 2016 in an early
-        #                2.x release.  Do not remove before Jan 2017
-        #                and a 3.x release.
-        project_group = parser.add_mutually_exclusive_group()
-        project_group.add_argument(
+        parser.add_argument(
             "--project",
             metavar="<project>",
-            help="Set an alternate project on this image (name or ID)",
-        )
-        project_group.add_argument(
-            "--owner",
-            metavar="<project>",
-            help=argparse.SUPPRESS,
+            help=_("Set an alternate project on this image (name or ID)"),
         )
         return parser
 
     def take_action(self, parsed_args):
         image_client = self.app.client_manager.image
 
-        if getattr(parsed_args, 'owner', None) is not None:
-            self.log.warning(_(
-                'The --owner option is deprecated, '
-                'please use --project instead.'
-            ))
-
         # Build an attribute dict from the parsed args, only include
         # attributes that were actually set on the command line
         kwargs = {}
@@ -211,7 +240,7 @@ class CreateImage(command.ShowOne):
         # Special case project option back to API attribute name 'owner'
         val = getattr(parsed_args, 'project', None)
         if val:
-            kwargs['owner'] = val
+            kwargs['owner_id'] = val
 
         # Handle exclusive booleans with care
         # Avoid including attributes in kwargs if an option is not
@@ -220,9 +249,9 @@ class CreateImage(command.ShowOne):
         # to do nothing when no options are present as opposed to always
         # setting a default.
         if parsed_args.protected:
-            kwargs['protected'] = True
+            kwargs['is_protected'] = True
         if parsed_args.unprotected:
-            kwargs['protected'] = False
+            kwargs['is_protected'] = False
         if parsed_args.public:
             kwargs['is_public'] = True
         if parsed_args.private:
@@ -251,30 +280,39 @@ class CreateImage(command.ShowOne):
                 kwargs["data"] = io.open(parsed_args.file, "rb")
             else:
                 # Read file from stdin
-                if sys.stdin.isatty() is not True:
+                if not sys.stdin.isatty():
                     if msvcrt:
                         msvcrt.setmode(sys.stdin.fileno(), os.O_BINARY)
-                    # Send an open file handle to glanceclient so it will
-                    # do a chunked transfer
-                    kwargs["data"] = sys.stdin
+                    if hasattr(sys.stdin, 'buffer'):
+                        kwargs['data'] = sys.stdin.buffer
+                    else:
+                        kwargs["data"] = sys.stdin
 
         if not parsed_args.volume:
             # Wrap the call to catch exceptions in order to close files
             try:
-                image = image_client.images.create(**kwargs)
+                image = image_client.create_image(**kwargs)
             finally:
                 # Clean up open files - make sure data isn't a string
                 if ('data' in kwargs and hasattr(kwargs['data'], 'close') and
                         kwargs['data'] != sys.stdin):
                     kwargs['data'].close()
 
+        if image:
+            display_columns, columns = _get_columns(image)
+            _formatters['properties'] = format_columns.DictColumn
+            data = utils.get_item_properties(image, columns,
+                                             formatters=_formatters)
+            return (display_columns, data)
+        elif info:
             info.update(image._info)
-            info['properties'] = utils.format_dict(info.get('properties', {}))
-        return zip(*sorted(six.iteritems(info)))
+            info['properties'] = format_columns.DictColumn(
+                info.get('properties', {}))
+            return zip(*sorted(info.items()))
 
 
 class DeleteImage(command.Command):
-    """Delete image(s)"""
+    _description = _("Delete image(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteImage, self).get_parser(prog_name)
@@ -282,22 +320,19 @@ class DeleteImage(command.Command):
             "images",
             metavar="<image>",
             nargs="+",
-            help="Image(s) to delete (name or ID)",
+            help=_("Image(s) to delete (name or ID)"),
         )
         return parser
 
     def take_action(self, parsed_args):
         image_client = self.app.client_manager.image
         for image in parsed_args.images:
-            image_obj = utils.find_resource(
-                image_client.images,
-                image,
-            )
-            image_client.images.delete(image_obj.id)
+            image_obj = image_client.find_image(image)
+            image_client.delete_image(image_obj.id)
 
 
 class ListImage(command.Lister):
-    """List available images"""
+    _description = _("List available images")
 
     def get_parser(self, prog_name):
         parser = super(ListImage, self).get_parser(prog_name)
@@ -307,14 +342,14 @@ class ListImage(command.Lister):
             dest="public",
             action="store_true",
             default=False,
-            help="List only public images",
+            help=_("List only public images"),
         )
         public_group.add_argument(
             "--private",
             dest="private",
             action="store_true",
             default=False,
-            help="List only private images",
+            help=_("List only private images"),
         )
         # Included for silent CLI compatibility with v2
         public_group.add_argument(
@@ -328,13 +363,13 @@ class ListImage(command.Lister):
             '--property',
             metavar='<key=value>',
             action=parseractions.KeyValueAction,
-            help='Filter output based on property',
+            help=_('Filter output based on property'),
         )
         parser.add_argument(
             '--long',
             action='store_true',
             default=False,
-            help='List additional fields in output',
+            help=_('List additional fields in output'),
         )
 
         # --page-size has never worked, leave here for silent compatibility
@@ -347,9 +382,10 @@ class ListImage(command.Lister):
         parser.add_argument(
             '--sort',
             metavar="<key>[:<direction>]",
-            help="Sort output by selected keys and directions(asc or desc) "
-                 "(default: asc), multiple keys and directions can be "
-                 "specified separated by comma",
+            default='name:asc',
+            help=_("Sort output by selected keys and directions(asc or desc) "
+                   "(default: name:asc), multiple keys and directions can be "
+                   "specified separated by comma"),
         )
         return parser
 
@@ -358,15 +394,9 @@ class ListImage(command.Lister):
 
         kwargs = {}
         if parsed_args.public:
-            kwargs['public'] = True
+            kwargs['is_public'] = True
         if parsed_args.private:
-            kwargs['private'] = True
-        # Note: We specifically need to do that below to get the 'status'
-        #       column.
-        #
-        # Always set kwargs['detailed'] to True, and then filter the columns
-        # according to whether the --long option is specified or not.
-        kwargs['detailed'] = True
+            kwargs['is_private'] = True
 
         if parsed_args.long:
             columns = (
@@ -375,10 +405,11 @@ class ListImage(command.Lister):
                 'Disk Format',
                 'Container Format',
                 'Size',
+                'Checksum',
                 'Status',
                 'is_public',
-                'protected',
-                'owner',
+                'is_protected',
+                'owner_id',
                 'properties',
             )
             column_headers = (
@@ -387,6 +418,7 @@ class ListImage(command.Lister):
                 'Disk Format',
                 'Container Format',
                 'Size',
+                'Checksum',
                 'Status',
                 'Visibility',
                 'Protected',
@@ -398,16 +430,7 @@ class ListImage(command.Lister):
             column_headers = columns
 
         # List of image data received
-        data = []
-        # No pages received yet, so start the page marker at None.
-        marker = None
-        while True:
-            page = image_client.api.image_list(marker=marker, **kwargs)
-            if not page:
-                break
-            data.extend(page)
-            # Set the marker to the id of the last item we received
-            marker = page[-1]['id']
+        data = list(image_client.images(**kwargs))
 
         if parsed_args.property:
             # NOTE(dtroyer): coerce to a list to subscript it in py3
@@ -423,202 +446,181 @@ class ListImage(command.Lister):
 
         return (
             column_headers,
-            (utils.get_dict_properties(
+            (utils.get_item_properties(
                 s,
                 columns,
                 formatters={
-                    'is_public': _format_visibility,
-                    'properties': utils.format_dict,
+                    'is_public': VisibilityColumn,
+                    'properties': format_columns.DictColumn,
                 },
             ) for s in data)
         )
 
 
 class SaveImage(command.Command):
-    """Save an image locally"""
+    _description = _("Save an image locally")
 
     def get_parser(self, prog_name):
         parser = super(SaveImage, self).get_parser(prog_name)
         parser.add_argument(
             "--file",
             metavar="<filename>",
-            help="Downloaded image save filename (default: stdout)",
+            help=_("Downloaded image save filename (default: stdout)"),
         )
         parser.add_argument(
             "image",
             metavar="<image>",
-            help="Image to save (name or ID)",
+            help=_("Image to save (name or ID)"),
         )
         return parser
 
     def take_action(self, parsed_args):
         image_client = self.app.client_manager.image
-        image = utils.find_resource(
-            image_client.images,
-            parsed_args.image,
-        )
-        data = image_client.images.data(image)
+        image = image_client.find_image(parsed_args.image)
 
-        gc_utils.save_image(data, parsed_args.file)
+        image_client.download_image(image.id, output=parsed_args.file)
 
 
 class SetImage(command.Command):
-    """Set image properties"""
+    _description = _("Set image properties")
 
     def get_parser(self, prog_name):
         parser = super(SetImage, self).get_parser(prog_name)
         parser.add_argument(
             "image",
             metavar="<image>",
-            help="Image to modify (name or ID)",
+            help=_("Image to modify (name or ID)"),
         )
         parser.add_argument(
             "--name",
             metavar="<name>",
-            help="New image name",
+            help=_("New image name"),
         )
         parser.add_argument(
             "--min-disk",
             metavar="<disk-gb>",
             type=int,
-            help="Minimum disk size needed to boot image, in gigabytes",
+            help=_("Minimum disk size needed to boot image, in gigabytes"),
         )
         parser.add_argument(
             "--min-ram",
             metavar="<disk-ram>",
             type=int,
-            help="Minimum RAM size needed to boot image, in megabytes",
+            help=_("Minimum RAM size needed to boot image, in megabytes"),
         )
-        container_choices = ["ami", "ari", "aki", "bare", "ovf"]
         parser.add_argument(
             "--container-format",
             metavar="<container-format>",
-            help=("Container format of image. Acceptable formats: %s" %
-                  container_choices),
-            choices=container_choices
+            choices=CONTAINER_CHOICES,
+            help=_("Image container format. The supported options are: %s") %
+            ', '.join(CONTAINER_CHOICES)
         )
-        disk_choices = ["ami", "ari", "aki", "vhd", "vmdk", "raw", "qcow2",
-                        "vdi", "iso"]
         parser.add_argument(
             "--disk-format",
             metavar="<disk-format>",
-            help="Disk format of image. Acceptable formats: %s" % disk_choices,
-            choices=disk_choices
+            choices=DISK_CHOICES,
+            help=_("Image disk format. The supported options are: %s.") %
+            ', '.join(DISK_CHOICES)
         )
         parser.add_argument(
             "--size",
             metavar="<size>",
             type=int,
-            help="Size of image data (in bytes)"
+            help=_("Size of image data (in bytes)")
         )
         protected_group = parser.add_mutually_exclusive_group()
         protected_group.add_argument(
             "--protected",
             action="store_true",
-            help="Prevent image from being deleted",
+            help=_("Prevent image from being deleted"),
         )
         protected_group.add_argument(
             "--unprotected",
             action="store_true",
-            help="Allow image to be deleted (default)",
+            help=_("Allow image to be deleted (default)"),
         )
         public_group = parser.add_mutually_exclusive_group()
         public_group.add_argument(
             "--public",
             action="store_true",
-            help="Image is accessible to the public",
+            help=_("Image is accessible to the public"),
         )
         public_group.add_argument(
             "--private",
             action="store_true",
-            help="Image is inaccessible to the public (default)",
+            help=_("Image is inaccessible to the public (default)"),
         )
         parser.add_argument(
             "--property",
             dest="properties",
             metavar="<key=value>",
             action=parseractions.KeyValueAction,
-            help="Set a property on this image "
-                 "(repeat option to set multiple properties)",
+            help=_("Set a property on this image "
+                   "(repeat option to set multiple properties)"),
         )
         parser.add_argument(
             "--store",
             metavar="<store>",
-            help="Upload image to this store",
+            help=_("Upload image to this store"),
         )
         parser.add_argument(
             "--location",
             metavar="<image-url>",
-            help="Download image from an existing URL",
+            help=_("Download image from an existing URL"),
         )
         parser.add_argument(
             "--copy-from",
             metavar="<image-url>",
-            help="Copy image from the data store (similar to --location)",
+            help=_("Copy image from the data store (similar to --location)"),
         )
         parser.add_argument(
             "--file",
             metavar="<file>",
-            help="Upload image from local file",
+            help=_("Upload image from local file"),
         )
         parser.add_argument(
             "--volume",
             metavar="<volume>",
-            help="Create image from a volume",
+            help=_("Create image from a volume"),
         )
         parser.add_argument(
             "--force",
             dest='force',
             action='store_true',
             default=False,
-            help="Force image change if volume is in use "
-            "(only meaningful with --volume)",
+            help=_("Force image change if volume is in use "
+                   "(only meaningful with --volume)"),
         )
         parser.add_argument(
             "--stdin",
             dest='stdin',
             action='store_true',
             default=False,
-            help="Read image data from standard input",
+            help=_("Read image data from standard input"),
         )
         parser.add_argument(
             "--checksum",
             metavar="<checksum>",
-            help="Image hash used for verification",
+            help=_("Image hash used for verification"),
         )
-        # NOTE(dtroyer): --owner is deprecated in Jan 2016 in an early
-        #                2.x release.  Do not remove before Jan 2017
-        #                and a 3.x release.
-        project_group = parser.add_mutually_exclusive_group()
-        project_group.add_argument(
+        parser.add_argument(
             "--project",
             metavar="<project>",
-            help="Set an alternate project on this image (name or ID)",
-        )
-        project_group.add_argument(
-            "--owner",
-            metavar="<project>",
-            help=argparse.SUPPRESS,
+            help=_("Set an alternate project on this image (name or ID)"),
         )
         return parser
 
     def take_action(self, parsed_args):
         image_client = self.app.client_manager.image
 
-        if getattr(parsed_args, 'owner', None) is not None:
-            self.log.warning(_(
-                'The --owner option is deprecated, '
-                'please use --project instead.'
-            ))
-
         kwargs = {}
         copy_attrs = ('name', 'owner', 'min_disk', 'min_ram', 'properties',
                       'container_format', 'disk_format', 'size', 'store',
-                      'location', 'copy_from', 'volume', 'force', 'checksum')
+                      'location', 'copy_from', 'volume', 'checksum')
         for attr in copy_attrs:
             if attr in parsed_args:
                 val = getattr(parsed_args, attr, None)
-                if val:
+                if val is not None:
                     # Only include a value in kwargs for attributes that are
                     # actually present on the command line
                     kwargs[attr] = val
@@ -635,9 +637,9 @@ class SetImage(command.Command):
         # to do nothing when no options are present as opposed to always
         # setting a default.
         if parsed_args.protected:
-            kwargs['protected'] = True
+            kwargs['is_protected'] = True
         if parsed_args.unprotected:
-            kwargs['protected'] = False
+            kwargs['is_protected'] = False
         if parsed_args.public:
             kwargs['is_public'] = True
         if parsed_args.private:
@@ -645,10 +647,7 @@ class SetImage(command.Command):
 
         # Wrap the call to catch exceptions in order to close files
         try:
-            image = utils.find_resource(
-                image_client.images,
-                parsed_args.image,
-            )
+            image = image_client.find_image(parsed_args.image)
 
             if not parsed_args.location and not parsed_args.copy_from:
                 if parsed_args.volume:
@@ -678,22 +677,19 @@ class SetImage(command.Command):
                         if parsed_args.stdin:
                             if msvcrt:
                                 msvcrt.setmode(sys.stdin.fileno(), os.O_BINARY)
-                            # Send an open file handle to glanceclient so it
-                            # will do a chunked transfer
-                            kwargs["data"] = sys.stdin
+                            if hasattr(sys.stdin, 'buffer'):
+                                kwargs['data'] = sys.stdin.buffer
+                            else:
+                                kwargs["data"] = sys.stdin
                         else:
-                            self.log.warning('Use --stdin to enable read image'
-                                             ' data from standard input')
+                            LOG.warning(_('Use --stdin to enable read image '
+                                          'data from standard input'))
 
             if image.properties and parsed_args.properties:
                 image.properties.update(kwargs['properties'])
                 kwargs['properties'] = image.properties
 
-            if not kwargs:
-                self.log.warning('no arguments specified')
-                return
-
-            image = image_client.images.update(image.id, **kwargs)
+            image = image_client.update_image(image.id, **kwargs)
         finally:
             # Clean up open files - make sure data isn't a string
             if ('data' in kwargs and hasattr(kwargs['data'], 'close') and
@@ -702,25 +698,31 @@ class SetImage(command.Command):
 
 
 class ShowImage(command.ShowOne):
-    """Display image details"""
+    _description = _("Display image details")
 
     def get_parser(self, prog_name):
         parser = super(ShowImage, self).get_parser(prog_name)
+        parser.add_argument(
+            "--human-readable",
+            default=False,
+            action='store_true',
+            help=_("Print image size in a human-friendly format."),
+        )
         parser.add_argument(
             "image",
             metavar="<image>",
-            help="Image to display (name or ID)",
+            help=_("Image to display (name or ID)"),
         )
         return parser
 
     def take_action(self, parsed_args):
         image_client = self.app.client_manager.image
-        image = utils.find_resource(
-            image_client.images,
-            parsed_args.image,
-        )
+        image = image_client.find_image(parsed_args.image)
 
-        info = {}
-        info.update(image._info)
-        info['properties'] = utils.format_dict(info.get('properties', {}))
-        return zip(*sorted(six.iteritems(info)))
+        if parsed_args.human_readable:
+            _formatters['size'] = HumanReadableSizeColumn
+        display_columns, columns = _get_columns(image)
+        _formatters['properties'] = format_columns.DictColumn
+        data = utils.get_item_properties(image, columns,
+                                         formatters=_formatters)
+        return (display_columns, data)
