@@ -16,31 +16,42 @@
 """Image V2 Action Implementations"""
 
 import argparse
+from base64 import b64encode
 import logging
+import os
+import sys
 
-from glanceclient.common import utils as gc_utils
+import openstack.cloud._utils
+from openstack.image import image_signer
+from osc_lib.api import utils as api_utils
+from osc_lib.cli import format_columns
 from osc_lib.cli import parseractions
 from osc_lib.command import command
 from osc_lib import exceptions
 from osc_lib import utils
-import six
 
-from openstackclient.api import utils as api_utils
+from openstackclient.common import sdk_utils
 from openstackclient.i18n import _
 from openstackclient.identity import common
 
+if os.name == "nt":
+    import msvcrt
+else:
+    msvcrt = None
+
 
 CONTAINER_CHOICES = ["ami", "ari", "aki", "bare", "docker", "ova", "ovf"]
 DEFAULT_CONTAINER_FORMAT = 'bare'
 DEFAULT_DISK_FORMAT = 'raw'
 DISK_CHOICES = ["ami", "ari", "aki", "vhd", "vmdk", "raw", "qcow2", "vhdx",
                 "vdi", "iso", "ploop"]
+MEMBER_STATUS_CHOICES = ["accepted", "pending", "rejected", "all"]
 
 
 LOG = logging.getLogger(__name__)
 
 
-def _format_image(image):
+def _format_image(image, human_readable=False):
     """Format an image to make it more consistent with OSC operations."""
 
     info = {}
@@ -52,25 +63,80 @@ def _format_image(image):
                       'min_disk', 'protected', 'id', 'file', 'checksum',
                       'owner', 'virtual_size', 'min_ram', 'schema']
 
+    # TODO(gtema/anybody): actually it should be possible to drop this method,
+    # since SDK already delivers a proper object
+    image = image.to_dict(ignore_none=True, original_names=True)
+
     # split out the usual key and the properties which are top-level
-    for key in six.iterkeys(image):
+    for key in image:
         if key in fields_to_show:
             info[key] = image.get(key)
         elif key == 'tags':
             continue  # handle this later
-        else:
+        elif key == 'properties':
+            # NOTE(gtema): flatten content of properties
+            properties.update(image.get(key))
+        elif key != 'location':
             properties[key] = image.get(key)
 
+    if human_readable:
+        info['size'] = utils.format_size(image['size'])
+
     # format the tags if they are there
-    info['tags'] = utils.format_list(image.get('tags'))
+    info['tags'] = format_columns.ListColumn(image.get('tags'))
 
     # add properties back into the dictionary as a top-level key
     if properties:
-        info['properties'] = utils.format_dict(properties)
+        info['properties'] = format_columns.DictColumn(properties)
 
     return info
 
 
+_formatters = {
+    'tags': format_columns.ListColumn,
+}
+
+
+def _get_member_columns(item):
+    # Trick sdk_utils to return URI attribute
+    column_map = {
+        'image_id': 'image_id'
+    }
+    hidden_columns = ['id', 'location', 'name']
+    return sdk_utils.get_osc_show_columns_for_sdk_resource(
+        item.to_dict(), column_map, hidden_columns)
+
+
+def get_data_file(args):
+    if args.file:
+        return (open(args.file, 'rb'), args.file)
+    else:
+        # distinguish cases where:
+        # (1) stdin is not valid (as in cron jobs):
+        #    openstack ... <&-
+        # (2) image data is provided through stdin:
+        #    openstack ... < /tmp/file
+        # (3) no image data provided
+        #    openstack ...
+        try:
+            os.fstat(0)
+        except OSError:
+            # (1) stdin is not valid
+            return (None, None)
+        if not sys.stdin.isatty():
+            # (2) image data is provided through stdin
+            image = sys.stdin
+            if hasattr(sys.stdin, 'buffer'):
+                image = sys.stdin.buffer
+            if msvcrt:
+                msvcrt.setmode(sys.stdin.fileno(), os.O_BINARY)
+
+            return (image, None)
+        else:
+            # (3)
+            return (None, None)
+
+
 class AddProjectToImage(command.ShowOne):
     _description = _("Associate project with image")
 
@@ -84,7 +150,7 @@ class AddProjectToImage(command.ShowOne):
         parser.add_argument(
             "project",
             metavar="<project>",
-            help=_("Project to associate with image (name or ID)"),
+            help=_("Project to associate with image (ID)"),
         )
         common.add_project_domain_option_to_parser(parser)
         return parser
@@ -93,20 +159,26 @@ class AddProjectToImage(command.ShowOne):
         image_client = self.app.client_manager.image
         identity_client = self.app.client_manager.identity
 
-        project_id = common.find_project(identity_client,
-                                         parsed_args.project,
-                                         parsed_args.project_domain).id
+        if openstack.cloud._utils._is_uuid_like(parsed_args.project):
+            project_id = parsed_args.project
+        else:
+            project_id = common.find_project(
+                identity_client,
+                parsed_args.project,
+                parsed_args.project_domain).id
 
-        image_id = utils.find_resource(
-            image_client.images,
-            parsed_args.image).id
+        image = image_client.find_image(parsed_args.image,
+                                        ignore_missing=False)
 
-        image_member = image_client.image_members.create(
-            image_id,
-            project_id,
+        obj = image_client.add_member(
+            image=image.id,
+            member_id=project_id,
         )
 
-        return zip(*sorted(six.iteritems(image_member)))
+        display_columns, columns = _get_member_columns(obj)
+        data = utils.get_item_properties(obj, columns, formatters={})
+
+        return (display_columns, data)
 
 
 class CreateImage(command.ShowOne):
@@ -183,6 +255,22 @@ class CreateImage(command.ShowOne):
             help=_("Force image creation if volume is in use "
                    "(only meaningful with --volume)"),
         )
+        parser.add_argument(
+            '--sign-key-path',
+            metavar="<sign-key-path>",
+            default=[],
+            help=_("Sign the image using the specified private key. "
+                   "Only use in combination with --sign-cert-id")
+        )
+        parser.add_argument(
+            '--sign-cert-id',
+            metavar="<sign-cert-id>",
+            default=[],
+            help=_("The specified certificate UUID is a reference to "
+                   "the certificate in the key manager that corresponds "
+                   "to the public key and is used for signature validation. "
+                   "Only use in combination with --sign-key-path")
+        )
         protected_group = parser.add_mutually_exclusive_group()
         protected_group.add_argument(
             "--protected",
@@ -231,19 +319,18 @@ class CreateImage(command.ShowOne):
             help=_("Set a tag on this image "
                    "(repeat option to set multiple tags)"),
         )
-        # NOTE(dtroyer): --owner is deprecated in Jan 2016 in an early
-        #                2.x release.  Do not remove before Jan 2017
-        #                and a 3.x release.
-        project_group = parser.add_mutually_exclusive_group()
-        project_group.add_argument(
+        parser.add_argument(
             "--project",
             metavar="<project>",
             help=_("Set an alternate project on this image (name or ID)"),
         )
-        project_group.add_argument(
-            "--owner",
-            metavar="<project>",
-            help=argparse.SUPPRESS,
+        parser.add_argument(
+            "--import",
+            dest="use_import",
+            action="store_true",
+            help=_(
+                "Force the use of glance image import instead of"
+                " direct upload")
         )
         common.add_project_domain_option_to_parser(parser)
         for deadopt in self.deadopts:
@@ -281,7 +368,7 @@ class CreateImage(command.ShowOne):
 
         # properties should get flattened into the general kwargs
         if getattr(parsed_args, 'properties', None):
-            for k, v in six.iteritems(parsed_args.properties):
+            for k, v in parsed_args.properties.items():
                 kwargs[k] = str(v)
 
         # Handle exclusive booleans with care
@@ -291,9 +378,9 @@ class CreateImage(command.ShowOne):
         # to do nothing when no options are present as opposed to always
         # setting a default.
         if parsed_args.protected:
-            kwargs['protected'] = True
+            kwargs['is_protected'] = True
         if parsed_args.unprotected:
-            kwargs['protected'] = False
+            kwargs['is_protected'] = False
         if parsed_args.public:
             kwargs['visibility'] = 'public'
         if parsed_args.private:
@@ -302,38 +389,81 @@ class CreateImage(command.ShowOne):
             kwargs['visibility'] = 'community'
         if parsed_args.shared:
             kwargs['visibility'] = 'shared'
-        # Handle deprecated --owner option
-        project_arg = parsed_args.project
-        if parsed_args.owner:
-            project_arg = parsed_args.owner
-            LOG.warning(_('The --owner option is deprecated, '
-                          'please use --project instead.'))
-        if project_arg:
-            kwargs['owner'] = common.find_project(
+        if parsed_args.project:
+            kwargs['owner_id'] = common.find_project(
                 identity_client,
-                project_arg,
+                parsed_args.project,
                 parsed_args.project_domain,
             ).id
 
+        if parsed_args.use_import:
+            kwargs['use_import'] = True
+
         # open the file first to ensure any failures are handled before the
-        # image is created
-        fp = gc_utils.get_data_file(parsed_args)
+        # image is created. Get the file name (if it is file, and not stdin)
+        # for easier further handling.
+        (fp, fname) = get_data_file(parsed_args)
         info = {}
+
         if fp is not None and parsed_args.volume:
             raise exceptions.CommandError(_("Uploading data and using "
                                             "container are not allowed at "
                                             "the same time"))
-
         if fp is None and parsed_args.file:
             LOG.warning(_("Failed to get an image file."))
             return {}, {}
+        elif fname:
+            kwargs['filename'] = fname
+        elif fp:
+            kwargs['validate_checksum'] = False
+            kwargs['data'] = fp
 
-        if parsed_args.owner:
-            kwargs['owner'] = common.find_project(
-                identity_client,
-                parsed_args.owner,
-                parsed_args.project_domain,
-            ).id
+        # sign an image using a given local private key file
+        if parsed_args.sign_key_path or parsed_args.sign_cert_id:
+            if not parsed_args.file:
+                msg = (_("signing an image requires the --file option, "
+                         "passing files via stdin when signing is not "
+                         "supported."))
+                raise exceptions.CommandError(msg)
+            if (len(parsed_args.sign_key_path) < 1 or
+                    len(parsed_args.sign_cert_id) < 1):
+                msg = (_("'sign-key-path' and 'sign-cert-id' must both be "
+                         "specified when attempting to sign an image."))
+                raise exceptions.CommandError(msg)
+            else:
+                sign_key_path = parsed_args.sign_key_path
+                sign_cert_id = parsed_args.sign_cert_id
+                signer = image_signer.ImageSigner()
+                try:
+                    pw = utils.get_password(
+                        self.app.stdin,
+                        prompt=("Please enter private key password, leave "
+                                "empty if none: "),
+                        confirm=False)
+
+                    if not pw or len(pw) < 1:
+                        pw = None
+                    else:
+                        # load_private_key() requires the password to be
+                        # passed as bytes
+                        pw = pw.encode()
+
+                    signer.load_private_key(
+                        sign_key_path,
+                        password=pw)
+                except Exception:
+                    msg = (_("Error during sign operation: private key "
+                             "could not be loaded."))
+                    raise exceptions.CommandError(msg)
+
+                signature = signer.generate_signature(fp)
+                signature_b64 = b64encode(signature)
+                kwargs['img_signature'] = signature_b64
+                kwargs['img_signature_certificate_uuid'] = sign_cert_id
+                kwargs['img_signature_hash_method'] = signer.hash_method
+                if signer.padding_method:
+                    kwargs['img_signature_key_type'] = \
+                        signer.padding_method
 
         # If a volume is specified.
         if parsed_args.volume:
@@ -355,31 +485,12 @@ class CreateImage(command.ShowOne):
             except TypeError:
                 info['volume_type'] = None
         else:
-            image = image_client.images.create(**kwargs)
-
-        if fp is not None:
-            with fp:
-                try:
-                    image_client.images.upload(image.id, fp)
-                except Exception:
-                    # If the upload fails for some reason attempt to remove the
-                    # dangling queued image made by the create() call above but
-                    # only if the user did not specify an id which indicates
-                    # the Image already exists and should be left alone.
-                    try:
-                        if 'id' not in kwargs:
-                            image_client.images.delete(image.id)
-                    except Exception:
-                        pass  # we don't care about this one
-                    raise  # now, throw the upload exception again
-
-                # update the image after the data has been uploaded
-                image = image_client.images.get(image.id)
+            image = image_client.create_image(**kwargs)
 
         if not info:
             info = _format_image(image)
 
-        return zip(*sorted(six.iteritems(info)))
+        return zip(*sorted(info.items()))
 
 
 class DeleteImage(command.Command):
@@ -401,11 +512,9 @@ class DeleteImage(command.Command):
         image_client = self.app.client_manager.image
         for image in parsed_args.images:
             try:
-                image_obj = utils.find_resource(
-                    image_client.images,
-                    image,
-                )
-                image_client.images.delete(image_obj.id)
+                image_obj = image_client.find_image(image,
+                                                    ignore_missing=False)
+                image_client.delete_image(image_obj.id)
             except Exception as e:
                 del_result += 1
                 LOG.error(_("Failed to delete image with name or "
@@ -457,7 +566,8 @@ class ListImage(command.Lister):
             '--property',
             metavar='<key=value>',
             action=parseractions.KeyValueAction,
-            help=_('Filter output based on property'),
+            help=_('Filter output based on property '
+                   '(repeat option to filter on multiple properties)'),
         )
         parser.add_argument(
             '--name',
@@ -471,6 +581,16 @@ class ListImage(command.Lister):
             default=None,
             help=_("Filter images based on status.")
         )
+        parser.add_argument(
+            '--member-status',
+            metavar='<member-status>',
+            default=None,
+            type=lambda s: s.lower(),
+            choices=MEMBER_STATUS_CHOICES,
+            help=(_("Filter images based on member status. "
+                    "The supported options are: %s. ") %
+                  ', '.join(MEMBER_STATUS_CHOICES))
+        )
         parser.add_argument(
             '--tag',
             metavar='<tag>',
@@ -520,22 +640,23 @@ class ListImage(command.Lister):
 
         kwargs = {}
         if parsed_args.public:
-            kwargs['public'] = True
+            kwargs['visibility'] = 'public'
         if parsed_args.private:
-            kwargs['private'] = True
+            kwargs['visibility'] = 'private'
         if parsed_args.community:
-            kwargs['community'] = True
+            kwargs['visibility'] = 'community'
         if parsed_args.shared:
-            kwargs['shared'] = True
+            kwargs['visibility'] = 'shared'
         if parsed_args.limit:
             kwargs['limit'] = parsed_args.limit
         if parsed_args.marker:
-            kwargs['marker'] = utils.find_resource(image_client.images,
-                                                   parsed_args.marker).id
+            kwargs['marker'] = image_client.find_image(parsed_args.marker).id
         if parsed_args.name:
             kwargs['name'] = parsed_args.name
         if parsed_args.status:
             kwargs['status'] = parsed_args.status
+        if parsed_args.member_status:
+            kwargs['member_status'] = parsed_args.member_status
         if parsed_args.tag:
             kwargs['tag'] = parsed_args.tag
         if parsed_args.long:
@@ -548,8 +669,8 @@ class ListImage(command.Lister):
                 'Checksum',
                 'Status',
                 'visibility',
-                'protected',
-                'owner',
+                'is_protected',
+                'owner_id',
                 'tags',
             )
             column_headers = (
@@ -570,40 +691,28 @@ class ListImage(command.Lister):
             column_headers = columns
 
         # List of image data received
-        data = []
-        if 'marker' in kwargs:
-            data = image_client.api.image_list(**kwargs)
-        else:
-            # No pages received yet, so start the page marker at None.
-            marker = None
-            while True:
-                page = image_client.api.image_list(marker=marker, **kwargs)
-                if not page:
-                    break
-                data.extend(page)
-                # Set the marker to the id of the last item we received
-                marker = page[-1]['id']
+        if 'limit' in kwargs:
+            # Disable automatic pagination in SDK
+            kwargs['paginated'] = False
+        data = list(image_client.images(**kwargs))
 
         if parsed_args.property:
-            # NOTE(dtroyer): coerce to a list to subscript it in py3
-            attr, value = list(parsed_args.property.items())[0]
-            api_utils.simple_filter(
-                data,
-                attr=attr,
-                value=value,
-                property_field='properties',
-            )
+            for attr, value in parsed_args.property.items():
+                api_utils.simple_filter(
+                    data,
+                    attr=attr,
+                    value=value,
+                    property_field='properties',
+                )
 
         data = utils.sort_items(data, parsed_args.sort, str)
 
         return (
             column_headers,
-            (utils.get_dict_properties(
+            (utils.get_item_properties(
                 s,
                 columns,
-                formatters={
-                    'tags': utils.format_list,
-                },
+                formatters=_formatters,
             ) for s in data)
         )
 
@@ -629,11 +738,9 @@ class ListImageProjects(command.Lister):
             "Status"
         )
 
-        image_id = utils.find_resource(
-            image_client.images,
-            parsed_args.image).id
+        image_id = image_client.find_image(parsed_args.image).id
 
-        data = image_client.image_members.list(image_id)
+        data = image_client.members(image=image_id)
 
         return (columns,
                 (utils.get_item_properties(
@@ -667,11 +774,12 @@ class RemoveProjectImage(command.Command):
                                          parsed_args.project,
                                          parsed_args.project_domain).id
 
-        image_id = utils.find_resource(
-            image_client.images,
-            parsed_args.image).id
+        image = image_client.find_image(parsed_args.image,
+                                        ignore_missing=False)
 
-        image_client.image_members.delete(image_id, project_id)
+        image_client.remove_member(
+            member=project_id,
+            image=image.id)
 
 
 class SaveImage(command.Command):
@@ -693,19 +801,9 @@ class SaveImage(command.Command):
 
     def take_action(self, parsed_args):
         image_client = self.app.client_manager.image
-        image = utils.find_resource(
-            image_client.images,
-            parsed_args.image,
-        )
-        data = image_client.images.data(image.id)
+        image = image_client.find_image(parsed_args.image)
 
-        if data.wrapped is None:
-            msg = _('Image %s has no data.') % image.id
-            LOG.error(msg)
-            self.app.stdout.write(msg + '\n')
-            raise SystemExit
-
-        gc_utils.save_image(data, parsed_args.file)
+        image_client.download_image(image.id, output=parsed_args.file)
 
 
 class SetImage(command.Command):
@@ -857,20 +955,11 @@ class SetImage(command.Command):
             action="store_true",
             help=_("Activate the image"),
         )
-        # NOTE(dtroyer): --owner is deprecated in Jan 2016 in an early
-        #                2.x release.  Do not remove before Jan 2017
-        #                and a 3.x release.
-        project_group = parser.add_mutually_exclusive_group()
-        project_group.add_argument(
+        parser.add_argument(
             "--project",
             metavar="<project>",
             help=_("Set an alternate project on this image (name or ID)"),
         )
-        project_group.add_argument(
-            "--owner",
-            metavar="<project>",
-            help=argparse.SUPPRESS,
-        )
         common.add_project_domain_option_to_parser(parser)
         for deadopt in self.deadopts:
             parser.add_argument(
@@ -923,7 +1012,7 @@ class SetImage(command.Command):
 
         # Properties should get flattened into the general kwargs
         if getattr(parsed_args, 'properties', None):
-            for k, v in six.iteritems(parsed_args.properties):
+            for k, v in parsed_args.properties.items():
                 kwargs[k] = str(v)
 
         # Handle exclusive booleans with care
@@ -933,9 +1022,9 @@ class SetImage(command.Command):
         # to do nothing when no options are present as opposed to always
         # setting a default.
         if parsed_args.protected:
-            kwargs['protected'] = True
+            kwargs['is_protected'] = True
         if parsed_args.unprotected:
-            kwargs['protected'] = False
+            kwargs['is_protected'] = False
         if parsed_args.public:
             kwargs['visibility'] = 'public'
         if parsed_args.private:
@@ -944,30 +1033,27 @@ class SetImage(command.Command):
             kwargs['visibility'] = 'community'
         if parsed_args.shared:
             kwargs['visibility'] = 'shared'
-        # Handle deprecated --owner option
-        project_arg = parsed_args.project
-        if parsed_args.owner:
-            project_arg = parsed_args.owner
-            LOG.warning(_('The --owner option is deprecated, '
-                          'please use --project instead.'))
         project_id = None
-        if project_arg:
+        if parsed_args.project:
             project_id = common.find_project(
                 identity_client,
-                project_arg,
+                parsed_args.project,
                 parsed_args.project_domain,
             ).id
-            kwargs['owner'] = project_id
+            kwargs['owner_id'] = project_id
 
-        image = utils.find_resource(
-            image_client.images, parsed_args.image)
+        image = image_client.find_image(parsed_args.image,
+                                        ignore_missing=False)
+
+        # image = utils.find_resource(
+        #     image_client.images, parsed_args.image)
 
         activation_status = None
         if parsed_args.deactivate:
-            image_client.images.deactivate(image.id)
+            image_client.deactivate_image(image.id)
             activation_status = "deactivated"
         if parsed_args.activate:
-            image_client.images.reactivate(image.id)
+            image_client.reactivate_image(image.id)
             activation_status = "activated"
 
         membership_group_args = ('accept', 'reject', 'pending')
@@ -982,15 +1068,15 @@ class SetImage(command.Command):
             # most one item in the membership_status list.
             if membership_status[0] != 'pending':
                 membership_status[0] += 'ed'  # Glance expects the past form
-            image_client.image_members.update(
-                image.id, project_id, membership_status[0])
+            image_client.update_member(
+                image=image.id, member=project_id, status=membership_status[0])
 
         if parsed_args.tags:
             # Tags should be extended, but duplicates removed
             kwargs['tags'] = list(set(image.tags).union(set(parsed_args.tags)))
 
         try:
-            image = image_client.images.update(image.id, **kwargs)
+            image = image_client.update_image(image.id, **kwargs)
         except Exception:
             if activation_status is not None:
                 LOG.info(_("Image %(id)s was %(status)s."),
@@ -1018,15 +1104,12 @@ class ShowImage(command.ShowOne):
 
     def take_action(self, parsed_args):
         image_client = self.app.client_manager.image
-        image = utils.find_resource(
-            image_client.images,
-            parsed_args.image,
-        )
-        if parsed_args.human_readable:
-            image['size'] = utils.format_size(image['size'])
 
-        info = _format_image(image)
-        return zip(*sorted(six.iteritems(info)))
+        image = image_client.find_image(parsed_args.image,
+                                        ignore_missing=False)
+
+        info = _format_image(image, parsed_args.human_readable)
+        return zip(*sorted(info.items()))
 
 
 class UnsetImage(command.Command):
@@ -1061,10 +1144,8 @@ class UnsetImage(command.Command):
 
     def take_action(self, parsed_args):
         image_client = self.app.client_manager.image
-        image = utils.find_resource(
-            image_client.images,
-            parsed_args.image,
-        )
+        image = image_client.find_image(parsed_args.image,
+                                        ignore_missing=False)
 
         kwargs = {}
         tagret = 0
@@ -1072,7 +1153,7 @@ class UnsetImage(command.Command):
         if parsed_args.tags:
             for k in parsed_args.tags:
                 try:
-                    image_client.image_tags.delete(image.id, k)
+                    image_client.remove_tag(image.id, k)
                 except Exception:
                     LOG.error(_("tag unset failed, '%s' is a "
                                 "nonexistent tag "), k)
@@ -1080,13 +1161,26 @@ class UnsetImage(command.Command):
 
         if parsed_args.properties:
             for k in parsed_args.properties:
-                if k not in image:
+                if k in image:
+                    kwargs[k] = None
+                elif k in image.properties:
+                    # Since image is an "evil" object from SDK POV we need to
+                    # pass modified properties object, so that SDK can figure
+                    # out, what was changed inside
+                    # NOTE: ping gtema to improve that in SDK
+                    new_props = kwargs.get('properties',
+                                           image.get('properties').copy())
+                    new_props.pop(k, None)
+                    kwargs['properties'] = new_props
+                else:
                     LOG.error(_("property unset failed, '%s' is a "
                                 "nonexistent property "), k)
                     propret += 1
-            image_client.images.update(
-                image.id,
-                parsed_args.properties,
+
+            # We must give to update a current image for the reference on what
+            # has changed
+            image_client.update_image(
+                image,
                 **kwargs)
 
         tagtotal = len(parsed_args.tags)
