@@ -18,37 +18,41 @@ import copy
 import json
 import logging
 
+from cliff import columns as cliff_columns
+from osc_lib.cli import format_columns
 from osc_lib.cli import parseractions
 from osc_lib.command import command
 from osc_lib import exceptions
 from osc_lib import utils
+from osc_lib.utils import tags as _tag
 
 from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
+from openstackclient.network import common
 from openstackclient.network import sdk_utils
-from openstackclient.network.v2 import _tag
 
 
 LOG = logging.getLogger(__name__)
 
 
-def _format_admin_state(state):
-    return 'UP' if state else 'DOWN'
+class AdminStateColumn(cliff_columns.FormattableColumn):
+    def human_readable(self):
+        return 'UP' if self._value else 'DOWN'
 
 
 _formatters = {
-    'admin_state_up': _format_admin_state,
-    'is_admin_state_up': _format_admin_state,
-    'allowed_address_pairs': utils.format_list_of_dicts,
-    'binding_profile': utils.format_dict,
-    'binding_vif_details': utils.format_dict,
-    'binding:profile': utils.format_dict,
-    'binding:vif_details': utils.format_dict,
-    'dns_assignment': utils.format_list_of_dicts,
-    'extra_dhcp_opts': utils.format_list_of_dicts,
-    'fixed_ips': utils.format_list_of_dicts,
-    'security_group_ids': utils.format_list,
-    'tags': utils.format_list,
+    'admin_state_up': AdminStateColumn,
+    'is_admin_state_up': AdminStateColumn,
+    'allowed_address_pairs': format_columns.ListDictColumn,
+    'binding_profile': format_columns.DictColumn,
+    'binding_vif_details': format_columns.DictColumn,
+    'binding:profile': format_columns.DictColumn,
+    'binding:vif_details': format_columns.DictColumn,
+    'dns_assignment': format_columns.ListDictColumn,
+    'extra_dhcp_opts': format_columns.ListDictColumn,
+    'fixed_ips': format_columns.ListDictColumn,
+    'security_group_ids': format_columns.ListColumn,
+    'tags': format_columns.ListColumn,
 }
 
 
@@ -95,21 +99,6 @@ class JSONKeyValueAction(argparse.Action):
 def _get_attrs(client_manager, parsed_args):
     attrs = {}
 
-    # Handle deprecated options
-    # NOTE(dtroyer): --device-id and --host-id were deprecated in Mar 2016.
-    #                Do not remove before 3.x release or Mar 2017.
-    if parsed_args.device_id:
-        attrs['device_id'] = parsed_args.device_id
-        LOG.warning(_(
-            'The --device-id option is deprecated, '
-            'please use --device instead.'
-        ))
-    if parsed_args.host_id:
-        attrs['binding:host_id'] = parsed_args.host_id
-        LOG.warning(_(
-            'The --host-id option is deprecated, '
-            'please use --host instead.'
-        ))
     if parsed_args.description is not None:
         attrs['description'] = parsed_args.description
     if parsed_args.device:
@@ -133,7 +122,7 @@ def _get_attrs(client_manager, parsed_args):
         attrs['dns_name'] = parsed_args.dns_name
     # It is possible that name is not updated during 'port set'
     if parsed_args.name is not None:
-        attrs['name'] = str(parsed_args.name)
+        attrs['name'] = parsed_args.name
     # The remaining options do not support 'port set' command, so they require
     # additional check
     if 'network' in parsed_args and parsed_args.network is not None:
@@ -162,6 +151,23 @@ def _get_attrs(client_manager, parsed_args):
         attrs['qos_policy_id'] = client_manager.network.find_qos_policy(
             parsed_args.qos_policy, ignore_missing=False).id
 
+    if ('enable_uplink_status_propagation' in parsed_args and
+            parsed_args.enable_uplink_status_propagation):
+        attrs['propagate_uplink_status'] = True
+    if ('disable_uplink_status_propagation' in parsed_args and
+            parsed_args.disable_uplink_status_propagation):
+        attrs['propagate_uplink_status'] = False
+
+    if ('numa_policy_required' in parsed_args and
+            parsed_args.numa_policy_required):
+        attrs['numa_affinity_policy'] = 'required'
+    elif ('numa_policy_preferred' in parsed_args and
+          parsed_args.numa_policy_preferred):
+        attrs['numa_affinity_policy'] = 'preferred'
+    elif ('numa_policy_legacy' in parsed_args and
+          parsed_args.numa_policy_legacy):
+        attrs['numa_affinity_policy'] = 'legacy'
+
     return attrs
 
 
@@ -227,19 +233,11 @@ def _add_updatable_args(parser):
         metavar='<description>',
         help=_("Description of this port")
     )
-    # NOTE(dtroyer): --device-id is deprecated in Mar 2016.  Do not
-    #                remove before 3.x release or Mar 2017.
-    device_group = parser.add_mutually_exclusive_group()
-    device_group.add_argument(
+    parser.add_argument(
         '--device',
         metavar='<device-id>',
         help=_("Port device ID")
     )
-    device_group.add_argument(
-        '--device-id',
-        metavar='<device-id>',
-        help=argparse.SUPPRESS,
-    )
     parser.add_argument(
         '--mac-address',
         metavar='<mac-address>',
@@ -258,21 +256,13 @@ def _add_updatable_args(parser):
                  'normal', 'baremetal', 'virtio-forwarder'],
         help=_("VNIC type for this port (direct | direct-physical | "
                "macvtap | normal | baremetal | virtio-forwarder, "
-               " default: normal)")
+               "default: normal)")
     )
-    # NOTE(dtroyer): --host-id is deprecated in Mar 2016.  Do not
-    #                remove before 3.x release or Mar 2017.
-    host_group = parser.add_mutually_exclusive_group()
-    host_group.add_argument(
+    parser.add_argument(
         '--host',
         metavar='<host-id>',
         help=_("Allocate port on host <host-id> (ID only)")
     )
-    host_group.add_argument(
-        '--host-id',
-        metavar='<host-id>',
-        help=argparse.SUPPRESS,
-    )
     parser.add_argument(
         '--dns-domain',
         metavar='dns-domain',
@@ -281,10 +271,26 @@ def _add_updatable_args(parser):
     )
     parser.add_argument(
         '--dns-name',
-        metavar='dns-name',
-        help=_("Set DNS name to this port "
+        metavar='<dns-name>',
+        help=_("Set DNS name for this port "
                "(requires DNS integration extension)")
     )
+    numa_affinity_policy_group = parser.add_mutually_exclusive_group()
+    numa_affinity_policy_group.add_argument(
+        '--numa-policy-required',
+        action='store_true',
+        help=_("NUMA affinity policy required to schedule this port")
+    )
+    numa_affinity_policy_group.add_argument(
+        '--numa-policy-preferred',
+        action='store_true',
+        help=_("NUMA affinity policy preferred to schedule this port")
+    )
+    numa_affinity_policy_group.add_argument(
+        '--numa-policy-legacy',
+        action='store_true',
+        help=_("NUMA affinity policy using legacy mode to schedule this port")
+    )
 
 
 # TODO(abhiraut): Use the SDK resource mapped attribute names once the
@@ -300,6 +306,19 @@ def _convert_address_pairs(parsed_args):
     return ops
 
 
+def _convert_extra_dhcp_options(parsed_args):
+    dhcp_options = []
+    for opt in parsed_args.extra_dhcp_options:
+        option = {}
+        option['opt_name'] = opt['name']
+        if 'value' in opt:
+            option['opt_value'] = opt['value']
+        if 'ip-version' in opt:
+            option['ip_version'] = opt['ip-version']
+        dhcp_options.append(option)
+    return dhcp_options
+
+
 class CreatePort(command.ShowOne):
     _description = _("Create a new port")
 
@@ -348,6 +367,17 @@ class CreatePort(command.ShowOne):
             action='store_true',
             help=_("Disable port")
         )
+        uplink_status_group = parser.add_mutually_exclusive_group()
+        uplink_status_group.add_argument(
+            '--enable-uplink-status-propagation',
+            action='store_true',
+            help=_("Enable uplink status propagate")
+        )
+        uplink_status_group.add_argument(
+            '--disable-uplink-status-propagation',
+            action='store_true',
+            help=_("Disable uplink status propagate (default)")
+        )
         parser.add_argument(
             '--project',
             metavar='<project>',
@@ -359,8 +389,18 @@ class CreatePort(command.ShowOne):
             metavar='<name>',
             help=_("Name of this port")
         )
-        # TODO(singhj): Add support for extended options:
-        # dhcp
+        parser.add_argument(
+            '--extra-dhcp-option',
+            metavar='name=<name>[,value=<value>,ip-version={4,6}]',
+            default=[],
+            action=parseractions.MultiKeyValueCommaAction,
+            dest='extra_dhcp_options',
+            required_keys=['name'],
+            optional_keys=['value', "ip-version"],
+            help=_('Extra DHCP options to be assigned to this port: '
+                   'name=<name>[,value=<value>,ip-version={4,6}] '
+                   '(repeat option to set multiple extra DHCP options)'))
+
         secgroups = parser.add_mutually_exclusive_group()
         secgroups.add_argument(
             '--security-group',
@@ -434,12 +474,29 @@ class CreatePort(command.ShowOne):
             attrs['allowed_address_pairs'] = (
                 _convert_address_pairs(parsed_args))
 
+        if parsed_args.extra_dhcp_options:
+            attrs["extra_dhcp_opts"] = _convert_extra_dhcp_options(parsed_args)
+
         if parsed_args.qos_policy:
             attrs['qos_policy_id'] = client.find_qos_policy(
                 parsed_args.qos_policy, ignore_missing=False).id
-        obj = client.create_port(**attrs)
-        # tags cannot be set when created, so tags need to be set later.
-        _tag.update_tags_for_set(client, obj, parsed_args)
+
+        set_tags_in_post = bool(
+            client.find_extension('tag-ports-during-bulk-creation'))
+        if set_tags_in_post:
+            if parsed_args.no_tag:
+                attrs['tags'] = []
+            if parsed_args.tags:
+                attrs['tags'] = list(set(parsed_args.tags))
+
+        with common.check_missing_extension_if_error(
+                self.app.client_manager.network, attrs):
+            obj = client.create_port(**attrs)
+
+        if not set_tags_in_post:
+            # tags cannot be set when created, so tags need to be set later.
+            _tag.update_tags_for_set(client, obj, parsed_args)
+
         display_columns, columns = _get_columns(obj)
         data = utils.get_item_properties(obj, columns, formatters=_formatters)
 
@@ -494,6 +551,10 @@ class ListPort(command.Lister):
                    "This is the entity that uses the port (for example, "
                    "network:dhcp).")
         )
+        parser.add_argument(
+            '--host',
+            metavar='<host-id>',
+            help=_("List only ports bound to this host ID"))
         parser.add_argument(
             '--network',
             metavar='<network>',
@@ -582,6 +643,8 @@ class ListPort(command.Lister):
             server = utils.find_resource(compute_client.servers,
                                          parsed_args.server)
             filters['device_id'] = server.id
+        if parsed_args.host:
+            filters['binding:host_id'] = parsed_args.host
         if parsed_args.network:
             network = network_client.find_network(parsed_args.network,
                                                   ignore_missing=False)
@@ -665,7 +728,7 @@ class SetPort(command.Command):
         parser.add_argument(
             '--no-binding-profile',
             action='store_true',
-            help=_("Clear existing information of binding:profile."
+            help=_("Clear existing information of binding:profile. "
                    "Specify both --binding-profile and --no-binding-profile "
                    "to overwrite the current binding:profile information.")
         )
@@ -719,9 +782,9 @@ class SetPort(command.Command):
             '--no-allowed-address',
             dest='no_allowed_address_pair',
             action='store_true',
-            help=_("Clear existing allowed-address pairs associated"
-                   "with this port."
-                   "(Specify both --allowed-address and --no-allowed-address"
+            help=_("Clear existing allowed-address pairs associated "
+                   "with this port. "
+                   "(Specify both --allowed-address and --no-allowed-address "
                    "to overwrite the current allowed-address pairs)")
         )
         parser.add_argument(
@@ -790,7 +853,9 @@ class SetPort(command.Command):
             attrs['data_plane_status'] = parsed_args.data_plane_status
 
         if attrs:
-            client.update_port(obj, **attrs)
+            with common.check_missing_extension_if_error(
+                    self.app.client_manager.network, attrs):
+                client.update_port(obj, **attrs)
 
         # tags is a subresource and it needs to be updated separately.
         _tag.update_tags_for_set(client, obj, parsed_args)
@@ -837,7 +902,7 @@ class UnsetPort(command.Command):
             '--binding-profile',
             metavar='<binding-profile-key>',
             action='append',
-            help=_("Desired key which should be removed from binding:profile"
+            help=_("Desired key which should be removed from binding:profile "
                    "(repeat option to unset multiple binding:profile data)"))
         parser.add_argument(
             '--security-group',
@@ -861,8 +926,8 @@ class UnsetPort(command.Command):
             required_keys=['ip-address'],
             optional_keys=['mac-address'],
             help=_("Desired allowed-address pair which should be removed "
-                   "from this port: ip-address=<ip-address> "
-                   "[,mac-address=<mac-address>] (repeat option to set "
+                   "from this port: ip-address=<ip-address>"
+                   "[,mac-address=<mac-address>] (repeat option to unset "
                    "multiple allowed-address pairs)")
         )
         parser.add_argument(
@@ -876,6 +941,11 @@ class UnsetPort(command.Command):
             action='store_true',
             help=_("Clear existing information of data plane status")
         )
+        parser.add_argument(
+            '--numa-policy',
+            action='store_true',
+            help=_("Clear existing NUMA affinity policy")
+        )
 
         _tag.add_tag_option_to_parser_for_unset(parser, _('port'))
 
@@ -931,6 +1001,8 @@ class UnsetPort(command.Command):
             attrs['qos_policy_id'] = None
         if parsed_args.data_plane_status:
             attrs['data_plane_status'] = None
+        if parsed_args.numa_policy:
+            attrs['numa_affinity_policy'] = None
 
         if attrs:
             client.update_port(obj, **attrs)
