@@ -14,9 +14,9 @@
 #
 
 import copy
+from unittest import mock
 import uuid
 
-import mock
 from novaclient import api_versions
 
 from openstackclient.api import compute_v2
@@ -505,7 +505,7 @@ class FakeSecurityGroup(object):
 
     @staticmethod
     def get_security_groups(security_groups=None, count=2):
-        """Get an iterable MagicMock object with a list of faked security groups.
+        """Get an iterable MagicMock with a list of faked security groups.
 
         If security groups list is provided, then initialize the Mock object
         with the list. Otherwise create one.
@@ -1244,7 +1244,7 @@ class FakeServerGroup(object):
     """Fake one server group"""
 
     @staticmethod
-    def create_one_server_group(attrs=None):
+    def _create_one_server_group(attrs=None):
         """Create a fake server group
 
         :param Dictionary attrs:
@@ -1261,7 +1261,6 @@ class FakeServerGroup(object):
             'members': [],
             'metadata': {},
             'name': 'server-group-name-' + uuid.uuid4().hex,
-            'policies': [],
             'project_id': 'server-group-project-id-' + uuid.uuid4().hex,
             'user_id': 'server-group-user-id-' + uuid.uuid4().hex,
         }
@@ -1274,6 +1273,38 @@ class FakeServerGroup(object):
             loaded=True)
         return server_group
 
+    @staticmethod
+    def create_one_server_group(attrs=None):
+        """Create a fake server group
+
+        :param Dictionary attrs:
+            A dictionary with all attributes
+        :return:
+            A FakeResource object, with id and other attributes
+        """
+        if attrs is None:
+            attrs = {}
+        attrs.setdefault('policies', ['policy1', 'policy2'])
+        return FakeServerGroup._create_one_server_group(attrs)
+
+
+class FakeServerGroupV264(object):
+    """Fake one server group fo API >= 2.64"""
+
+    @staticmethod
+    def create_one_server_group(attrs=None):
+        """Create a fake server group
+
+        :param Dictionary attrs:
+            A dictionary with all attributes
+        :return:
+            A FakeResource object, with id and other attributes
+        """
+        if attrs is None:
+            attrs = {}
+        attrs.setdefault('policy', 'policy1')
+        return FakeServerGroup._create_one_server_group(attrs)
+
 
 class FakeUsage(object):
     """Fake one or more usage."""
@@ -1304,6 +1335,7 @@ class FakeUsage(object):
                     'local_gb': 1,
                     'memory_mb': 512,
                     'name': 'usage-name-' + uuid.uuid4().hex,
+                    'instance_id': uuid.uuid4().hex,
                     'state': 'active',
                     'uptime': 3600,
                     'vcpus': 1
@@ -1400,6 +1432,38 @@ class FakeQuota(object):
 
         return quota
 
+    @staticmethod
+    def create_one_comp_detailed_quota(attrs=None):
+        """Create one quota"""
+
+        attrs = attrs or {}
+
+        quota_attrs = {
+            'id': 'project-id-' + uuid.uuid4().hex,
+            'cores': {'reserved': 0, 'in_use': 0, 'limit': 20},
+            'fixed_ips': {'reserved': 0, 'in_use': 0, 'limit': 30},
+            'injected_files': {'reserved': 0, 'in_use': 0, 'limit': 100},
+            'injected_file_content_bytes': {
+                'reserved': 0, 'in_use': 0, 'limit': 10240},
+            'injected_file_path_bytes': {
+                'reserved': 0, 'in_use': 0, 'limit': 255},
+            'instances': {'reserved': 0, 'in_use': 0, 'limit': 50},
+            'key_pairs': {'reserved': 0, 'in_use': 0, 'limit': 20},
+            'metadata_items': {'reserved': 0, 'in_use': 0, 'limit': 10},
+            'ram': {'reserved': 0, 'in_use': 0, 'limit': 51200},
+            'server_groups': {'reserved': 0, 'in_use': 0, 'limit': 10},
+            'server_group_members': {'reserved': 0, 'in_use': 0, 'limit': 10}
+        }
+
+        quota_attrs.update(attrs)
+        quota = fakes.FakeResource(
+            info=copy.deepcopy(quota_attrs),
+            loaded=True)
+
+        quota.project_id = quota_attrs['id']
+
+        return quota
+
 
 class FakeLimits(object):
     """Fake limits"""
