@@ -12,9 +12,10 @@
 #
 
 import argparse
+from unittest import mock
+from unittest.mock import call
 
-import mock
-from mock import call
+from osc_lib.cli import format_columns
 from osc_lib import exceptions
 from osc_lib import utils
 
@@ -58,41 +59,47 @@ class TestPort(network_fakes.TestNetworkV2):
             'mac_address',
             'name',
             'network_id',
+            'numa_affinity_policy',
             'port_security_enabled',
             'project_id',
+            'qos_network_policy_id',
             'qos_policy_id',
             'security_group_ids',
             'status',
             'tags',
+            'uplink_status_propagation',
         )
 
         data = (
-            port._format_admin_state(fake_port.admin_state_up),
-            utils.format_list_of_dicts(fake_port.allowed_address_pairs),
+            port.AdminStateColumn(fake_port.admin_state_up),
+            format_columns.ListDictColumn(fake_port.allowed_address_pairs),
             fake_port.binding_host_id,
-            utils.format_dict(fake_port.binding_profile),
-            utils.format_dict(fake_port.binding_vif_details),
+            format_columns.DictColumn(fake_port.binding_profile),
+            format_columns.DictColumn(fake_port.binding_vif_details),
             fake_port.binding_vif_type,
             fake_port.binding_vnic_type,
             fake_port.data_plane_status,
             fake_port.description,
             fake_port.device_id,
             fake_port.device_owner,
-            utils.format_list_of_dicts(fake_port.dns_assignment),
+            format_columns.ListDictColumn(fake_port.dns_assignment),
             fake_port.dns_domain,
             fake_port.dns_name,
-            utils.format_list_of_dicts(fake_port.extra_dhcp_opts),
-            utils.format_list_of_dicts(fake_port.fixed_ips),
+            format_columns.ListDictColumn(fake_port.extra_dhcp_opts),
+            format_columns.ListDictColumn(fake_port.fixed_ips),
             fake_port.id,
             fake_port.mac_address,
             fake_port.name,
             fake_port.network_id,
+            fake_port.numa_affinity_policy,
             fake_port.port_security_enabled,
             fake_port.project_id,
+            fake_port.qos_network_policy_id,
             fake_port.qos_policy_id,
-            utils.format_list(fake_port.security_group_ids),
+            format_columns.ListColumn(fake_port.security_group_ids),
             fake_port.status,
-            utils.format_list(fake_port.tags),
+            format_columns.ListColumn(fake_port.tags),
+            fake_port.uplink_status_propagation,
         )
 
         return columns, data
@@ -114,6 +121,7 @@ class TestCreatePort(TestPort):
         self.network.find_network = mock.Mock(return_value=fake_net)
         self.fake_subnet = network_fakes.FakeSubnet.create_one_subnet()
         self.network.find_subnet = mock.Mock(return_value=self.fake_subnet)
+        self.network.find_extension = mock.Mock(return_value=[])
         # Get the command object to test
         self.cmd = port.CreatePort(self.app, self.namespace)
 
@@ -139,7 +147,7 @@ class TestCreatePort(TestPort):
         self.assertFalse(self.network.set_tags.called)
 
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, data)
+        self.assertItemEqual(self.data, data)
 
     def test_create_full_options(self):
         arglist = [
@@ -197,7 +205,7 @@ class TestCreatePort(TestPort):
         })
 
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, data)
+        self.assertItemEqual(self.data, data)
 
     def test_create_invalid_json_binding_profile(self):
         arglist = [
@@ -248,7 +256,7 @@ class TestCreatePort(TestPort):
         })
 
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, data)
+        self.assertItemEqual(self.data, data)
 
     def test_create_with_security_group(self):
         secgroup = network_fakes.FakeSecurityGroup.create_one_security_group()
@@ -277,7 +285,7 @@ class TestCreatePort(TestPort):
         })
 
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, data)
+        self.assertItemEqual(self.data, data)
 
     def test_create_port_with_dns_name(self):
         arglist = [
@@ -303,7 +311,7 @@ class TestCreatePort(TestPort):
         })
 
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, data)
+        self.assertItemEqual(self.data, data)
 
     def test_create_with_security_groups(self):
         sg_1 = network_fakes.FakeSecurityGroup.create_one_security_group()
@@ -333,7 +341,7 @@ class TestCreatePort(TestPort):
         })
 
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, data)
+        self.assertItemEqual(self.data, data)
 
     def test_create_with_no_security_groups(self):
         arglist = [
@@ -359,7 +367,7 @@ class TestCreatePort(TestPort):
         })
 
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, data)
+        self.assertItemEqual(self.data, data)
 
     def test_create_with_no_fixed_ips(self):
         arglist = [
@@ -385,7 +393,7 @@ class TestCreatePort(TestPort):
         })
 
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, data)
+        self.assertItemEqual(self.data, data)
 
     def test_create_port_with_allowed_address_pair_ipaddr(self):
         pairs = [{'ip_address': '192.168.1.123'},
@@ -415,7 +423,7 @@ class TestCreatePort(TestPort):
         })
 
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, data)
+        self.assertItemEqual(self.data, data)
 
     def test_create_port_with_allowed_address_pair(self):
         pairs = [{'ip_address': '192.168.1.123',
@@ -451,7 +459,7 @@ class TestCreatePort(TestPort):
         })
 
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, data)
+        self.assertItemEqual(self.data, data)
 
     def test_create_port_with_qos(self):
         qos_policy = network_fakes.FakeNetworkQosPolicy.create_one_qos_policy()
@@ -479,7 +487,7 @@ class TestCreatePort(TestPort):
         })
 
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, data)
+        self.assertItemEqual(self.data, data)
 
     def test_create_port_security_enabled(self):
         arglist = [
@@ -529,7 +537,7 @@ class TestCreatePort(TestPort):
             'name': 'test-port',
         })
 
-    def _test_create_with_tag(self, add_tags=True):
+    def _test_create_with_tag(self, add_tags=True, add_tags_in_post=True):
         arglist = [
             '--network', self._port.network_id,
             'test-port',
@@ -548,28 +556,182 @@ class TestCreatePort(TestPort):
         else:
             verifylist.append(('no_tag', True))
 
+        self.network.find_extension = mock.Mock(return_value=add_tags_in_post)
+
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         columns, data = (self.cmd.take_action(parsed_args))
 
-        self.network.create_port.assert_called_once_with(
-            admin_state_up=True,
-            network_id=self._port.network_id,
-            name='test-port'
-        )
-        if add_tags:
-            self.network.set_tags.assert_called_once_with(
-                self._port,
-                tests_utils.CompareBySet(['red', 'blue']))
+        args = {
+            'admin_state_up': True,
+            'network_id': self._port.network_id,
+            'name': 'test-port',
+        }
+        if add_tags_in_post:
+            if add_tags:
+                args['tags'] = sorted(['red', 'blue'])
+            else:
+                args['tags'] = []
+            self.network.create_port.assert_called_once()
+            # Now we need to verify if arguments to call create_port are as
+            # expected,
+            # But we can't simply use assert_called_once_with() method because
+            # duplicates from 'tags' are removed with
+            # list(set(parsed_args.tags)) and that don't quarantee order of
+            # tags list which is used to call create_port().
+            create_port_call_kwargs = self.network.create_port.call_args[1]
+            create_port_call_kwargs['tags'] = sorted(
+                create_port_call_kwargs['tags'])
+            self.assertDictEqual(args, create_port_call_kwargs)
         else:
-            self.assertFalse(self.network.set_tags.called)
+            self.network.create_port.assert_called_once_with(
+                admin_state_up=True,
+                network_id=self._port.network_id,
+                name='test-port'
+            )
+            if add_tags:
+                self.network.set_tags.assert_called_once_with(
+                    self._port,
+                    tests_utils.CompareBySet(['red', 'blue']))
+            else:
+                self.assertFalse(self.network.set_tags.called)
+
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, data)
+        self.assertItemEqual(self.data, data)
 
     def test_create_with_tags(self):
-        self._test_create_with_tag(add_tags=True)
+        self._test_create_with_tag(add_tags=True, add_tags_in_post=True)
 
     def test_create_with_no_tag(self):
-        self._test_create_with_tag(add_tags=False)
+        self._test_create_with_tag(add_tags=False, add_tags_in_post=True)
+
+    def test_create_with_tags_using_put(self):
+        self._test_create_with_tag(add_tags=True, add_tags_in_post=False)
+
+    def test_create_with_no_tag_using_put(self):
+        self._test_create_with_tag(add_tags=False, add_tags_in_post=False)
+
+    def _test_create_with_uplink_status_propagation(self, enable=True):
+        arglist = [
+            '--network', self._port.network_id,
+            'test-port',
+        ]
+        if enable:
+            arglist += ['--enable-uplink-status-propagation']
+        else:
+            arglist += ['--disable-uplink-status-propagation']
+        verifylist = [
+            ('network', self._port.network_id,),
+            ('name', 'test-port'),
+        ]
+        if enable:
+            verifylist.append(('enable_uplink_status_propagation', True))
+        else:
+            verifylist.append(('disable_uplink_status_propagation', True))
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = (self.cmd.take_action(parsed_args))
+
+        self.network.create_port.assert_called_once_with(**{
+            'admin_state_up': True,
+            'network_id': self._port.network_id,
+            'propagate_uplink_status': enable,
+            'name': 'test-port',
+        })
+
+        self.assertEqual(self.columns, columns)
+        self.assertItemEqual(self.data, data)
+
+    def test_create_with_uplink_status_propagation_enabled(self):
+        self._test_create_with_uplink_status_propagation(enable=True)
+
+    def test_create_with_uplink_status_propagation_disabled(self):
+        self._test_create_with_uplink_status_propagation(enable=False)
+
+    def test_create_port_with_extra_dhcp_option(self):
+        extra_dhcp_options = [{'opt_name': 'classless-static-route',
+                               'opt_value': '169.254.169.254/32,22.2.0.2,'
+                                            '0.0.0.0/0,22.2.0.1',
+                               'ip_version': '4'},
+                              {'opt_name': 'dns-server',
+                               'opt_value': '240C::6666',
+                               'ip_version': '6'}]
+        arglist = [
+            '--network', self._port.network_id,
+            '--extra-dhcp-option', 'name=classless-static-route,'
+                                   'value=169.254.169.254/32,22.2.0.2,'
+                                   '0.0.0.0/0,22.2.0.1,'
+                                   'ip-version=4',
+            '--extra-dhcp-option', 'name=dns-server,value=240C::6666,'
+                                   'ip-version=6',
+            'test-port',
+        ]
+
+        verifylist = [
+            ('network', self._port.network_id,),
+            ('extra_dhcp_options', [{'name': 'classless-static-route',
+                                     'value': '169.254.169.254/32,22.2.0.2,'
+                                              '0.0.0.0/0,22.2.0.1',
+                                     'ip-version': '4'},
+                                    {'name': 'dns-server',
+                                     'value': '240C::6666',
+                                     'ip-version': '6'}]),
+            ('name', 'test-port'),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+
+        self.network.create_port.assert_called_once_with(**{
+            'admin_state_up': True,
+            'network_id': self._port.network_id,
+            'extra_dhcp_opts': extra_dhcp_options,
+            'name': 'test-port',
+        })
+
+    def _test_create_with_numa_affinity_policy(self, policy=None):
+        arglist = [
+            '--network', self._port.network_id,
+            'test-port',
+        ]
+        if policy:
+            arglist += ['--numa-policy-%s' % policy]
+
+        numa_affinity_policy = None if not policy else policy
+        verifylist = [
+            ('network', self._port.network_id,),
+            ('name', 'test-port'),
+        ]
+        if policy:
+            verifylist.append(('numa_policy_%s' % policy, True))
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = (self.cmd.take_action(parsed_args))
+
+        create_args = {
+            'admin_state_up': True,
+            'network_id': self._port.network_id,
+            'name': 'test-port',
+        }
+        if numa_affinity_policy:
+            create_args['numa_affinity_policy'] = numa_affinity_policy
+        self.network.create_port.assert_called_once_with(**create_args)
+
+        self.assertEqual(self.columns, columns)
+        self.assertItemEqual(self.data, data)
+
+    def test_create_with_numa_affinity_policy_required(self):
+        self._test_create_with_numa_affinity_policy(policy='required')
+
+    def test_create_with_numa_affinity_policy_preferred(self):
+        self._test_create_with_numa_affinity_policy(policy='preferred')
+
+    def test_create_with_numa_affinity_policy_legacy(self):
+        self._test_create_with_numa_affinity_policy(policy='legacy')
+
+    def test_create_with_numa_affinity_policy_null(self):
+        self._test_create_with_numa_affinity_policy()
 
 
 class TestDeletePort(TestPort):
@@ -680,7 +842,7 @@ class TestListPort(TestPort):
             prt.id,
             prt.name,
             prt.mac_address,
-            utils.format_list_of_dicts(prt.fixed_ips),
+            format_columns.ListDictColumn(prt.fixed_ips),
             prt.status,
         ))
 
@@ -690,11 +852,11 @@ class TestListPort(TestPort):
             prt.id,
             prt.name,
             prt.mac_address,
-            utils.format_list_of_dicts(prt.fixed_ips),
+            format_columns.ListDictColumn(prt.fixed_ips),
             prt.status,
-            utils.format_list(prt.security_group_ids),
+            format_columns.ListColumn(prt.security_group_ids),
             prt.device_owner,
-            utils.format_list(prt.tags),
+            format_columns.ListColumn(prt.tags),
         ))
 
     def setUp(self):
@@ -723,7 +885,7 @@ class TestListPort(TestPort):
 
         self.network.ports.assert_called_once_with()
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, list(data))
+        self.assertListItemEqual(self.data, list(data))
 
     def test_port_list_router_opt(self):
         arglist = [
@@ -742,7 +904,7 @@ class TestListPort(TestPort):
             'device_id': 'fake-router-id'
         })
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, list(data))
+        self.assertListItemEqual(self.data, list(data))
 
     @mock.patch.object(utils, 'find_resource')
     def test_port_list_with_server_option(self, mock_find):
@@ -762,7 +924,7 @@ class TestListPort(TestPort):
             device_id=fake_server.id)
         mock_find.assert_called_once_with(mock.ANY, 'fake-server-name')
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, list(data))
+        self.assertListItemEqual(self.data, list(data))
 
     def test_port_list_device_id_opt(self):
         arglist = [
@@ -781,7 +943,7 @@ class TestListPort(TestPort):
             'device_id': self._ports[0].device_id
         })
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, list(data))
+        self.assertListItemEqual(self.data, list(data))
 
     def test_port_list_device_owner_opt(self):
         arglist = [
@@ -800,7 +962,7 @@ class TestListPort(TestPort):
             'device_owner': self._ports[0].device_owner
         })
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, list(data))
+        self.assertListItemEqual(self.data, list(data))
 
     def test_port_list_all_opt(self):
         arglist = [
@@ -828,7 +990,7 @@ class TestListPort(TestPort):
             'mac_address': self._ports[0].mac_address
         })
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, list(data))
+        self.assertListItemEqual(self.data, list(data))
 
     def test_port_list_mac_address_opt(self):
         arglist = [
@@ -847,7 +1009,7 @@ class TestListPort(TestPort):
             'mac_address': self._ports[0].mac_address
         })
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, list(data))
+        self.assertListItemEqual(self.data, list(data))
 
     def test_port_list_fixed_ip_opt_ip_address(self):
         ip_address = self._ports[0].fixed_ips[0]['ip_address']
@@ -865,7 +1027,7 @@ class TestListPort(TestPort):
         self.network.ports.assert_called_once_with(**{
             'fixed_ips': ['ip_address=%s' % ip_address]})
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, list(data))
+        self.assertListItemEqual(self.data, list(data))
 
     def test_port_list_fixed_ip_opt_ip_address_substr(self):
         ip_address_ss = self._ports[0].fixed_ips[0]['ip_address'][:-1]
@@ -883,7 +1045,7 @@ class TestListPort(TestPort):
         self.network.ports.assert_called_once_with(**{
             'fixed_ips': ['ip_address_substr=%s' % ip_address_ss]})
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, list(data))
+        self.assertListItemEqual(self.data, list(data))
 
     def test_port_list_fixed_ip_opt_subnet_id(self):
         subnet_id = self._ports[0].fixed_ips[0]['subnet_id']
@@ -903,7 +1065,7 @@ class TestListPort(TestPort):
         self.network.ports.assert_called_once_with(**{
             'fixed_ips': ['subnet_id=%s' % subnet_id]})
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, list(data))
+        self.assertListItemEqual(self.data, list(data))
 
     def test_port_list_fixed_ip_opts(self):
         subnet_id = self._ports[0].fixed_ips[0]['subnet_id']
@@ -927,7 +1089,7 @@ class TestListPort(TestPort):
             'fixed_ips': ['subnet_id=%s' % subnet_id,
                           'ip_address=%s' % ip_address]})
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, list(data))
+        self.assertListItemEqual(self.data, list(data))
 
     def test_port_list_fixed_ips(self):
         subnet_id = self._ports[0].fixed_ips[0]['subnet_id']
@@ -951,7 +1113,7 @@ class TestListPort(TestPort):
             'fixed_ips': ['subnet_id=%s' % subnet_id,
                           'ip_address=%s' % ip_address]})
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, list(data))
+        self.assertListItemEqual(self.data, list(data))
 
     def test_list_port_with_long(self):
         arglist = [
@@ -968,7 +1130,23 @@ class TestListPort(TestPort):
 
         self.network.ports.assert_called_once_with()
         self.assertEqual(self.columns_long, columns)
-        self.assertEqual(self.data_long, list(data))
+        self.assertListItemEqual(self.data_long, list(data))
+
+    def test_port_list_host(self):
+        arglist = [
+            '--host', 'foobar',
+        ]
+        verifylist = [
+            ('host', 'foobar'),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+        filters = {'binding:host_id': 'foobar'}
+
+        self.network.ports.assert_called_once_with(**filters)
+        self.assertEqual(self.columns, columns)
+        self.assertListItemEqual(self.data, list(data))
 
     def test_port_list_project(self):
         project = identity_fakes.FakeProject.create_one_project()
@@ -986,7 +1164,7 @@ class TestListPort(TestPort):
 
         self.network.ports.assert_called_once_with(**filters)
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, list(data))
+        self.assertListItemEqual(self.data, list(data))
 
     def test_port_list_project_domain(self):
         project = identity_fakes.FakeProject.create_one_project()
@@ -1006,7 +1184,7 @@ class TestListPort(TestPort):
 
         self.network.ports.assert_called_once_with(**filters)
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, list(data))
+        self.assertListItemEqual(self.data, list(data))
 
     def test_list_with_tag_options(self):
         arglist = [
@@ -1031,7 +1209,7 @@ class TestListPort(TestPort):
                'not_any_tags': 'black,white'}
         )
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, list(data))
+        self.assertListItemEqual(self.data, list(data))
 
 
 class TestSetPort(TestPort):
@@ -1568,6 +1746,32 @@ class TestSetPort(TestPort):
     def test_set_with_no_tag(self):
         self._test_set_tags(with_tags=False)
 
+    def _test_create_with_numa_affinity_policy(self, policy):
+        arglist = [
+            '--numa-policy-%s' % policy,
+            self._port.id,
+        ]
+        verifylist = [
+            ('numa_policy_%s' % policy, True),
+            ('port', self._port.id,)
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+
+        self.network.update_port.assert_called_once_with(
+            self._port, **{'numa_affinity_policy': policy})
+
+    def test_create_with_numa_affinity_policy_required(self):
+        self._test_create_with_numa_affinity_policy('required')
+
+    def test_create_with_numa_affinity_policy_preferred(self):
+        self._test_create_with_numa_affinity_policy('preferred')
+
+    def test_create_with_numa_affinity_policy_legacy(self):
+        self._test_create_with_numa_affinity_policy('legacy')
+
 
 class TestShowPort(TestPort):
 
@@ -1605,7 +1809,7 @@ class TestShowPort(TestPort):
             self._port.name, ignore_missing=False)
 
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, data)
+        self.assertItemEqual(self.data, data)
 
 
 class TestUnsetPort(TestPort):
@@ -1823,3 +2027,26 @@ class TestUnsetPort(TestPort):
 
     def test_unset_with_all_tag(self):
         self._test_unset_tags(with_tags=False)
+
+    def test_unset_numa_affinity_policy(self):
+        _fake_port = network_fakes.FakePort.create_one_port(
+            {'numa_affinity_policy': 'required'})
+        self.network.find_port = mock.Mock(return_value=_fake_port)
+        arglist = [
+            '--numa-policy',
+            _fake_port.name,
+        ]
+        verifylist = [
+            ('numa_policy', True),
+            ('port', _fake_port.name),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        attrs = {
+            'numa_affinity_policy': None,
+        }
+
+        self.network.update_port.assert_called_once_with(_fake_port, **attrs)
+        self.assertIsNone(result)
